# -*- coding: utf-8 -*-

from ..outils.decimaux import decimaux
from ..outils.Arithmetique import carrerise, pgcd, ppcm
from math import sqrt
from Fractions import Fractions

def produitfacteurs(facteurs):
    """Affiche sous forme de produit les éléments d'une liste."""
    prodfacteurs = ''
    for element in facteurs:
        prodfacteurs += str(element) + ' \\times '
    return prodfacteurs[:-7]

class Racine:
    def __init__(self, radicande, coeff = 1, indice = 2):
        if (radicande < 0) or not (isinstance(indice, int)):
            print "Erreur de définition ! Le radicande doit être positif et l'indice un nombre entier !"
        self.radicande = radicande
        self.indice = indice # Nombre entier
        self.coeff = coeff # Coeff devant la racine

    def __add__(self, other):
        if not (isinstance(other, Racine)):
            return str(self) + " + " + str(other)
        if (self.radicande == other.radicande) and (self.indice == other.indice):
            return [str(self.simplifie()[0]) + "+" + str(other.simplifie()[0]), Racine(self.radicande, self.coeff + other.coeff, self.indice)]
        else:
            return str(self) + " + " + str(other)

    def __radd__(self, other):
        return self + other

    def __sub__(self, other):
        if not (isinstance(other, Racine)):
            return str(self) + " + " + str(other)
        if (self.radicande == other.radicande) and (self.indice == other.indice):
            return Racine(self.radicande, self.coeff - other.coeff, self.indice)
        else:
            return (str(self) + " - " + str(other), "tset")

    def __rsub__(self, other):
        return -(self - other)

    def __mul__(self, other):
        if (isinstance(other, float)) or (isinstance(other, int)):
            return Racine(self.radicande, self.coeff * other, self.indice)
        elif self.indice == other.indice:
            return Racine(self.radicande * other.radicande, self.coeff * other.coeff, self.indice)
        else:
            return str(self) + ' x ' + str(other)

    def __rmul__(self, other):
        return self * other

    def __div__(self, other):
        if (isinstance(other, float)) or (isinstance(other, int)):
            return Racine(self.radicande, self.coeff / other, self.indice)
        elif self.indice == other.indice:
            return Racine(self.radicande / float(other.radicande), self.coeff / float(other.coeff), self.indice)
        else:
            return str(self) + ' / ' + str(other)

    def __rdiv__(self, other):
        if (isinstance(other, float)) or (isinstance(other, int)):
            return Racine(self.radicande, other / float(self.coeff * self.radicande), self.indice)
        elif self.indice == other.indice:
            return Racine(other.radicande / float(self.radicande), other.coeff / float(self.coeff), self.indice)
        else:
            return str(other) + ' / ' + str(self)

    def __str__(self):
        if self.coeff == 0:
            return "0"
        else:
            if self.coeff == 1:
                coeff = ""
            else:
                coeff = decimaux(self.coeff, 1) + " \\, "
            if self.radicande == 0:
                coeff = ""
                racine = ""
                radicande = "0"
            elif self.indice == 2:
                racine = "\\sqrt{"
                radicande = decimaux(self.radicande, 1) + "}"
            else:
                racine = "\\sqrt[{0}]{{".format(self.indice)
                radicande = decimaux(self.radicande, 1) + "}"
            return coeff + racine + radicande

    def simplifie(self):
        facteurs = factor(self.radicande)
        prodfacteurs = produitfacteurs(facteurs)
        if self.indice == 2:
            racine = "\\sqrt{"
        else:
            racine = "\\sqrt[{0}]{{".format(self.indice)
        coeff = self.coeff
        radicande = self.radicande
        detail = [str(self), ' = ', str(coeff) + racine + prodfacteurs + '}']
        for element in facteurs:
            if facteurs.count(element) % self.indice == 0:
                coeff *= element
                for n in range(self.indice):
                    facteurs.remove(element)
                radicande = radicande // (element**(self.indice))
                prodfacteurs = produitfacteurs(facteurs)
                detail.append(' = ')
                detail.append(str(coeff) + ' \\times ' + racine + prodfacteurs + '}')
        if radicande == 1:
            return (coeff, detail)
        else:
            return (Racine(radicande, coeff, self.indice), detail)

def simplifie_racine(n):
    '''renvoie coeff,radicande où sqrt(n)=coeff*sqrt(radicande)'''
    if n==0:
        return 0,0
    else:
        ncar=carrerise(n)
        return int(sqrt(n//ncar)),ncar


def pTeX(n):
    """renvoie (n) si n<0"""
    if n<0:
        return "("+decimaux(n)+")"
    else:
        return decimaux(n)
def tTeX(n):
    if n==1:
        return ""
    elif n==-1:
        return "-"
    elif n>=0:
        return "+"+decimaux(n)
    else:
            return decimaux(n)



class RacineDegre2:
    def __init__(self,numerateur=0,denominateur=1,coeff=1,radicande=0):
        self.numerateur=numerateur
        self.denominateur=denominateur
        self.coeff=coeff
        self.radicande=radicande

    def __str__(self):
        if isinstance(self.numerateur,str):
            #utilisé dans le détail de la simplification
            numerateur=self.numerateur
        else:
            numerateur=decimaux(self.numerateur)*(self.numerateur!=0)   #renvoie "" si self.numerateur=0
        if self.radicande!=0:
            if isinstance(self.coeff,str):#utilisé dans le détail de la simplification
                if self.coeff[0]!="-" and self.coeff[0]!="+":
                    numerateur+="+"
                numerateur+=self.coeff+"\\sqrt{"+decimaux(self.radicande)+"}"
            elif self.coeff==1:
                numerateur+="+\\sqrt{"+decimaux(self.radicande)+"}"
            elif self.coeff==-1:
                numerateur+="-\\sqrt{"+decimaux(self.radicande)+"}"
            else:
                numerateur+=tTeX(self.coeff)+"\\sqrt{"+decimaux(self.radicande)+"}"
        if numerateur=='':
            numerateur = '0'
        if self.denominateur==1:
            result=numerateur
        else:
            result="\\dfrac{%s}{%s}"%(numerateur,self.denominateur)

        if result[0]=='+':
            return result[1:]
        else:
            return result

    def simplifie(self,detail=False):
        liste_detail=[]
        coeff,radicande=simplifie_racine(self.radicande)
        numerateur=self.numerateur
        if self.radicande!=0:
            if self.coeff==1:
                    det_coeff="+ "
            elif self.coeff==-1:
                det_coeff="- "
            else:
                det_coeff="%s\\times "%(self.coeff)
            if coeff!=1 or radicande==1:
                det_coeff+=str(coeff)
        else:
            det_coeff="0"
        if radicande==1:
            #det_coeff="%s\\times%s"%(tTeX(self.coeff),coeff)
            liste_detail.append("\\dfrac{%s %s}{%s}"%\
                                    (self.numerateur,det_coeff ,self.denominateur))
            radicande=0
            numerateur=self.numerateur+(self.coeff)*int(coeff)
            coeff=0

        if coeff!=1:
            liste_detail.append(str(RacineDegre2(numerateur,
                                                 self.denominateur,
                                                 det_coeff,
                                                 radicande)))

        coeff=(self.coeff)*int(coeff)

        simplifie=pgcd(pgcd(coeff,numerateur),self.denominateur)
        numerateur=numerateur//simplifie
        coeff=coeff//simplifie
        denominateur=self.denominateur//simplifie
        if simplifie!=1:
            if radicande!=0 or denominateur!=1:
                det_numerateur="%s_{\\times %s}"%(numerateur,pTeX(simplifie))
                det_denominateur="%s_{\\times %s}"%(denominateur,pTeX(simplifie))
                det_coeff="%s_{\\times %s}"%(coeff,pTeX(simplifie))
                liste_detail.append(str(RacineDegre2(det_numerateur,det_denominateur,det_coeff,radicande)))
            liste_detail.append(str(RacineDegre2(numerateur,denominateur,coeff,radicande)))
        if detail:
            return RacineDegre2(numerateur,denominateur,coeff,radicande),liste_detail
        return RacineDegre2(numerateur,denominateur,coeff,radicande)

    def __add__(self,other):
        if isinstance(other,RacineDegre2):
            if self.radicande==other.radicande or self.radicande==0 or other.radicande==0:
                radicande=max(self.radicande,other.radicande)
                premier,second=self,other
            else:
                premier=self.simplifie()
                second=other.simplifie()
                if self.radicande==other.radicande or self.radicande==0 or other.radicande==0:
                    radicande=max(self.radicande,other.radicande)
                else:
                    return NotImplemented

            denominateur=ppcm(premier.denominateur,second.denominateur)
            facteur1=denominateur/premier.denominateur
            facteur2=denominateur/second.denominateur
            #if self.radicande==other.radicande:
            coeff=premier.coeff*facteur1*(premier.radicande!=0)+second.coeff*facteur2*(second.radicande!=0)
            if coeff==0:
                radicande=0
            return RacineDegre2(premier.numerateur*facteur1+second.numerateur*facteur2,
                                denominateur,
                                coeff,
                                radicande)
        elif isinstance(other,int):
            return self + RacineDegre2(other)
        elif isinstance(other,Fractions):
            return self + RacineDegre2(other.numerateur,other.denominateur)

    def __radd__(self,other):
        return self+other
    def __neg__(self):
        return RacineDegre2(-self.numerateur,self.denominateur,-self.coeff,self.radicande)

    def __sub__(self,other):
        return self+(-other)

    def __rsub__(self,other):
        return -self + other

    def __mul__(self,other):
        if isinstance(other,Fractions):
            return self*RacineDegre2(other.numerateur,other.denominateur,0,self.radicande)
        elif isinstance(other,int):
            return self*RacineDegre2(other,1,0,self.radicande)
        else:
            radicande=max(self.radicande,other.radicande)#cela autorise d'avoir un radicande=0 mais n'efface pas l'autre
            coeff=self.numerateur*(other.coeff*(other.radicande!=0))+(self.coeff*(self.radicande!=0))*other.numerateur
            if coeff==0:
                radicande=0
            numerateur=self.numerateur*other.numerateur+(self.coeff*other.coeff)*(not(self.radicande==0 or other.radicande==0))*self.radicande
            return RacineDegre2(numerateur,
                            self.denominateur*other.denominateur,
                            coeff,
                            radicande)
    def __invert__(self):
        return self.denominateur*RacineDegre2(self.numerateur,self.numerateur**2-self.coeff**2*self.radicande,-self.coeff,self.radicande)
    def __div__(self,othrer):
        return self*~other
    def __rdiv__(self,other):
        return ~self*other
    def __rmul__(self,other):
        return self*other
    def __pow__(self,n):
        result=1
        for i in range(n):
            result=result*self
        return result
    def __float__(self):
        return (self.numerateur+self.coeff*sqrt(self.radicande))/self.denominateur
    def __cmp__(self,other):
        comp=float(self)-float(other)
        if comp>0:
            return 1
        elif comp<0:
            return -1
        else:
            return 0




#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if notPopen, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

from PyQt4 import QtGui, QtCore
import os, lxml, codecs, sys
from outils import System
from Values import HOME, CONFIGDIR, DATADIR, LESFICHES, COPYRIGHTS, VERSION, ICONDIR

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        self.configfile = os.path.join(CONFIGDIR,  "pyromaths.xml")
        self.liste_creation=[]
        if sys.platform != "darwin":  #Cas de Mac OS X.
            MainWindow.setStyleSheet("background-color: rgb(251, 245, 225);")
            MainWindow.setWindowIcon(QtGui.QIcon(ICONDIR))
        MainWindow.setWindowTitle("Pyromaths")
        MainWindow.setGeometry(0,44, 500, 200)
        font = QtGui.QFont()
        font.setPointSize(10)
        MainWindow.setFont(font)

        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Preferred)
        self.centralwidget = QtGui.QWidget(MainWindow)
        self.gridLayout = QtGui.QGridLayout(self.centralwidget)
        self.verticalLayout = QtGui.QVBoxLayout()
        self.verticalLayout.setMargin(9)

        #============================================================
        #        lecture du fichier de configuration
        #============================================================
        self.config = self.lire_config('options')

        #============================================================
        #        Boutons créer, quitter et annuler
        #============================================================
        self.pushButton_ok = QtGui.QPushButton(self.centralwidget)
        self.verticalLayout.addWidget(self.pushButton_ok)
        self.pushButton_ok.setText(u"Créer")
        if sys.platform != "darwin":  #Cas de Mac OS X.
            self.pushButton_ok.setStyleSheet("background-color: qlineargradient(spread:pad, x1:0, y1:0, x2:0, y2:1, stop:0 rgba(243, 165, 30, 255), stop:1 rgba(255, 247, 177, 255));")

        self.pushButton_quit = QtGui.QPushButton(self.centralwidget)
        self.verticalLayout.addWidget(self.pushButton_quit)
        self.pushButton_quit.setText("Quitter")
        if sys.platform != "darwin":  #Cas de Mac OS X.
            self.pushButton_quit.setStyleSheet("background-color: qlineargradient(spread:pad, x1:0, y1:0, x2:0, y2:1, stop:0 rgba(243, 165, 30, 255), stop:1 rgba(255, 247, 177, 255));")

        self.pushButton_erase = QtGui.QPushButton(self.centralwidget)
        self.verticalLayout.addWidget(self.pushButton_erase)
        self.pushButton_erase.setText(u"Réinitialiser")
        if sys.platform != "darwin":  #Cas de Mac OS X.
            self.pushButton_erase.setStyleSheet("background-color: qlineargradient(spread:pad, x1:0, y1:0, x2:0, y2:1, stop:0 rgba(243, 165, 30, 255), stop:1 rgba(255, 247, 177, 255));")


        spacerItem = QtGui.QSpacerItem(20, 40, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem)

        #============================================================
        #        Onglets de la zone centrale
        #============================================================

        self.gridLayout.addLayout(self.verticalLayout, 0, 1, 1, 1)
        self.tabWidget = QtGui.QTabWidget(self.centralwidget)
        self.tabWidget.setAutoFillBackground(True)
        if sys.platform != "darwin":  #Cas de Mac OS X.
            self.tabWidget.setStyleSheet("background-color: rgb(251, 231, 178);")

        #============================================================
        #        Remplissage des 4 niveaux
        #============================================================

        for level in range(5):
            exec("self.tab_%se = QtGui.QWidget()" % (6-level))
            exec("self.gridLayout_%se = QtGui.QGridLayout(self.tab_%se)" % (6-level, 6-level))
            nb_exos = len(LESFICHES[level][2])
            for i in range(nb_exos):
                self.insert_spinbox(6-level, i)
                for col in range(2):
                    exec("spacerItem_%s_%s = QtGui.QSpacerItem(20, 0, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)" % (6-level, col))
                    exec("self.gridLayout_%se.addItem(spacerItem_%s_%s, %s, %s, 1, 1)" % (6-level, 6-level, col, (nb_exos+1)/2, col))
            exec("self.tabWidget.addTab(self.tab_%se, \"\")" % (6-level))

        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_6e), "6e")
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_5e), "5e")
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_4e), "4e")
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_3e), "3e")
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_2e), u"Lycée")

        #============================================================
        #        Onglet options
        #============================================================

        ############## Onglet options

        self.tab_options = QtGui.QWidget()
        self.tabWidget.addTab(self.tab_options, "")
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_options), "Options")
        self.gridLayout_2 = QtGui.QGridLayout(self.tab_options)
        self.horizontalLayout_options1 = QtGui.QHBoxLayout()

        ############## Layout pour les noms d'options, en haut à gauche

        self.verticalLayout_16 = QtGui.QVBoxLayout()

        ############## Label nom du fichier

        self.opt_nom_fichier = QtGui.QLabel(self.tab_options)
        self.opt_nom_fichier.setText(u"Nom par défaut du fichier : ")
        self.verticalLayout_16.addWidget(self.opt_nom_fichier)

        ############## Label chemin par défaut pour l'enregistrement des fichiers

        self.opt_chemin_fichier = QtGui.QLabel(self.tab_options)
        self.opt_chemin_fichier.setText(u"Chemin par défaut pour enregistrer les fichiers : ")
        self.verticalLayout_16.addWidget(self.opt_chemin_fichier)

        ############## Label titre des fiches

        self.opt_titre_fiche = QtGui.QLabel(self.tab_options)
        self.opt_titre_fiche.setText("Titre de la fiche d'exercices : ")
        self.verticalLayout_16.addWidget(self.opt_titre_fiche)
        self.horizontalLayout_options1.addLayout(self.verticalLayout_16)

        ############## Layout pour les noms d'options, en haut à droite

        self.verticalLayout_17 = QtGui.QVBoxLayout()

        ############## LineEdit nom du fichier

        self.nom_fichier = QtGui.QLineEdit(self.tab_options)
        self.nom_fichier.setText(self.config['nom_fichier'])
        self.verticalLayout_17.addWidget(self.nom_fichier)

        ############## LineEdit chemin par défaut pour l'enregistrement des fichiers

        self.horizontalLayout_chemin_fichier = QtGui.QHBoxLayout()
        self.chemin_fichier = QtGui.QLineEdit(self.tab_options)
        self.chemin_fichier.setText(self.config['chemin_fichier'])
        self.horizontalLayout_chemin_fichier.addWidget(self.chemin_fichier)

        ############## Bouton parcourir

        self.pushButton_parcourir = QtGui.QPushButton(self.tab_options)
        self.pushButton_parcourir.setText("Parcourir")
        self.horizontalLayout_chemin_fichier.addWidget(self.pushButton_parcourir)
        self.verticalLayout_17.addLayout(self.horizontalLayout_chemin_fichier)

        ############## LineEdit titre des fiches

        self.titre_fiche = QtGui.QLineEdit(self.tab_options)
        self.titre_fiche.setText(self.config['titre_fiche'])
        self.verticalLayout_17.addWidget(self.titre_fiche)
        self.horizontalLayout_options1.addLayout(self.verticalLayout_17)
        self.gridLayout_2.addLayout(self.horizontalLayout_options1, 0, 0, 1, 2)

        ############## Ligne de séparation

        self.line = QtGui.QFrame(self.tab_options)
        self.line.setFrameShape(QtGui.QFrame.HLine)
        self.line.setFrameShadow(QtGui.QFrame.Sunken)
        self.gridLayout_2.addWidget(self.line, 1, 0, 1, 2)
        self.horizontalLayout_3 = QtGui.QHBoxLayout()
        self.verticalLayout_21 = QtGui.QVBoxLayout()

        ############## CheckBox "corrigés ou non"

        self.checkBox_corrige = QtGui.QCheckBox(self.tab_options)
        self.checkBox_corrige.setText(u"Créer le corrigé")
        self.checkBox_corrige.setToolTip(u"Pyromaths doit-il créer la fiche de correction détaillée?")
        self.checkBox_corrige.setChecked(int(self.config['corrige']))
        self.verticalLayout_21.addWidget(self.checkBox_corrige)

        ############## CheckBox "pdf ou non"

        self.checkBox_pdf = QtGui.QCheckBox(self.tab_options)
        self.checkBox_pdf.setText(u"Créer le pdf")
        self.checkBox_pdf.setToolTip(u"Pyromaths doit-il créer les fiches au format pdf ?")
        self.checkBox_pdf.setChecked(int(self.config['pdf']))
        self.verticalLayout_21.addWidget(self.checkBox_pdf)

        ############## CheckBox "pdf ou non"

        self.checkBox_unpdf = QtGui.QCheckBox(self.tab_options)
        self.checkBox_unpdf.setText(u"Créer un seul pdf")
        self.checkBox_unpdf.setToolTip(u"Le corrigé et les exercices doivent-ils être dans le même document ?")
        self.checkBox_unpdf.setChecked(int(self.config['unpdf']))
        self.verticalLayout_21.addWidget(self.checkBox_unpdf)

        ############## Espace

        self.horizontalLayout_3.addLayout(self.verticalLayout_21)
        spacerItem13 = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem13)
        self.verticalLayout_20 = QtGui.QVBoxLayout()
        self.horizontalLayout_2 = QtGui.QHBoxLayout()
        self.verticalLayout_18 = QtGui.QVBoxLayout()

        ############## Label niveau

        self.opt_niveau = QtGui.QLabel(self.tab_options)
        self.opt_niveau.setText("Niveau :")
        self.verticalLayout_18.addWidget(self.opt_niveau)

        ############## Label Modèle

        self.label_modele = QtGui.QLabel(self.tab_options)
        self.label_modele.setText(u"Modèle de mise en page :")
        #self.label_modele.setEnabled(False)
        self.verticalLayout_18.addWidget(self.label_modele)
        self.horizontalLayout_2.addLayout(self.verticalLayout_18)

        ############## Layout pour les noms d'options, en bas à droite

        self.verticalLayout_19 = QtGui.QVBoxLayout()

        ############## ComboBox niveau

        self.comboBox_niveau = QtGui.QComboBox(self.tab_options)
        self.comboBox_niveau.setEditable(True) # l’utilisateur peut entrer son propre texte
        self.comboBox_niveau.addItem("Classe de 6\\ieme")
        self.comboBox_niveau.addItem("Classe de 5\\ieme")
        self.comboBox_niveau.addItem("Classe de 4\\ieme")
        self.comboBox_niveau.addItem("Classe de 3\\ieme")
        self.comboBox_niveau.addItem("Classe de 2\\up{nde}")
        self.verticalLayout_19.addWidget(self.comboBox_niveau)

        ############## ComboBox modèles

        self.comboBox_modele = QtGui.QComboBox(self.tab_options)

        modeles = os.listdir(os.path.join(DATADIR, 'templates'))
        modeles_home = os.listdir(os.path.join(CONFIGDIR, 'templates'))

        count = 0
        for element in modeles:
          if os.path.splitext(element)[1] == ".tex":
             self.comboBox_modele.addItem(str(element[:len(element)-4]))
             if element == self.config['modele']:
               self.comboBox_modele.setCurrentIndex(count)
             count += 1


        for element in modeles_home:
          if os.path.splitext(element)[1] == ".tex":
             self.comboBox_modele.addItem(QtCore.QString())
             self.comboBox_modele.setItemText(count, str(element[:len(element)-4]))
             if element == self.config['modele']:
               self.comboBox_modele.setCurrentIndex(count)
             count += 1

        self.verticalLayout_19.addWidget(self.comboBox_modele)
        self.horizontalLayout_2.addLayout(self.verticalLayout_19)
        self.verticalLayout_20.addLayout(self.horizontalLayout_2)
        self.horizontalLayout_3.addLayout(self.verticalLayout_20)
        self.gridLayout_2.addLayout(self.horizontalLayout_3, 2, 0, 1, 2)

        ############## Bouton enregistrer

        self.pushButton_enr_opt = QtGui.QPushButton(self.tab_options)
        self.pushButton_enr_opt.setText(u"Enregistrer dans les préférences")
        if sys.platform != "darwin":  #Cas de Mac OS X.
            self.pushButton_enr_opt.setStyleSheet("background-color: qlineargradient(spread:pad, x1:0, y1:0, x2:0, y2:1, stop:0 rgba(243, 165, 30, 255), stop:1 rgba(255, 247, 177, 255));")

        self.gridLayout_2.addWidget(self.pushButton_enr_opt, 4, 1, 1, 1)
        spacerItem14 = QtGui.QSpacerItem(20, 177, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.gridLayout_2.addItem(spacerItem14, 3, 1, 1, 1)
        spacerItem15 = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.gridLayout_2.addItem(spacerItem15, 4, 0, 1, 1)

        #============================================================
        #        Barre de menus et de status
        #============================================================
        self.gridLayout.addWidget(self.tabWidget, 0, 0, 1, 1)
        MainWindow.setCentralWidget(self.centralwidget)

        self.menubar = QtGui.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 700, 22))
        if sys.platform != "darwin":  #Cas de Mac OS X.
            self.menubar.setStyleSheet("background-color: rgb(251, 231, 178);")
        MainWindow.setMenuBar(self.menubar)

        self.menuFichier = QtGui.QMenu(self.menubar)
        self.menuFichier.setTitle("Fichier")

        self.menu_propos = QtGui.QMenu(self.menubar)
        self.menu_propos.setTitle("Aide")

        self.statusbar = QtGui.QStatusBar(MainWindow)
        MainWindow.setStatusBar(self.statusbar)
        self.statusbar.showMessage(u"Pour avoir un aperçu d'un exercice, positionner le curseur de la souris sur le point d'interrogation.")
        if sys.platform != "darwin":  #Cas de Mac OS X.
            self.statusbar.setStyleSheet("background-color: rgb(251, 231, 178);")

        #============================================================
        #        Menus de la barre de menus
        #============================================================
        self.actionTous_les_exercices = QtGui.QAction(MainWindow)
        self.actionTous_les_exercices.setText("Tous les exercices")

        self.actionQuitter = QtGui.QAction(MainWindow)
        self.actionQuitter.setText("Quitter")

        self.actionAcceder_au_site = QtGui.QAction(MainWindow)
        self.actionAcceder_au_site.setText(u"Accéder au site")

        self.action_a_propos = QtGui.QAction(MainWindow)
        self.action_a_propos.setText(u"À propos")

        self.menuFichier.addAction(self.actionTous_les_exercices)
        self.menuFichier.addSeparator()
        self.menuFichier.addAction(self.actionQuitter)
        self.menu_propos.addAction(self.actionAcceder_au_site)
        self.menu_propos.addSeparator()
        self.menu_propos.addAction(self.action_a_propos)
        self.menubar.addAction(self.menuFichier.menuAction())
        self.menubar.addAction(self.menu_propos.menuAction())

        #============================================================
        #    Raccourcis clavier
        #============================================================
        keyQuit = QtGui.QShortcut("Ctrl+Q", MainWindow)
        QtCore.QObject.connect(keyQuit, QtCore.SIGNAL("activated()"), QtGui.qApp,
                                                                 QtCore.SLOT("quit()"))

        #============================================================
        #    Actions des boutons et menus
        #============================================================
        QtCore.QObject.connect(self.actionTous_les_exercices, QtCore.SIGNAL("activated()"), self.creer_tous_les_exercices)
        QtCore.QObject.connect(self.actionQuitter, QtCore.SIGNAL("activated()"), QtGui.qApp,
                                                                 QtCore.SLOT("quit()"))
        QtCore.QObject.connect(self.actionAcceder_au_site, QtCore.SIGNAL("activated()"), self.site)
        QtCore.QObject.connect(self.action_a_propos, QtCore.SIGNAL("activated()"), self.about)
        QtCore.QObject.connect(self.pushButton_quit, QtCore.SIGNAL("clicked()"), QtGui.qApp,
                                                                 QtCore.SLOT("quit()"))
        QtCore.QObject.connect(self.pushButton_erase, QtCore.SIGNAL("clicked()"), self.effacer_choix_exercices)
        QtCore.QObject.connect(self.pushButton_ok,QtCore.SIGNAL("clicked()"), self.creer_les_exercices)
        QtCore.QObject.connect(self.pushButton_enr_opt,QtCore.SIGNAL("clicked()"), self.enregistrer_config)
        QtCore.QObject.connect(self.pushButton_parcourir,QtCore.SIGNAL("clicked()"), self.option_parcourir)
        QtCore.QObject.connect(self.checkBox_corrige,QtCore.SIGNAL("stateChanged(int)"), self.option_corrige)
        #============================================================
        #        Actions des spinBox
        #============================================================
        for level in range(5):
            for box in range(len(LESFICHES[level][2])):
                exec("QtCore.QObject.connect(self.spinBox_%s_%s, QtCore.SIGNAL(\"valueChanged(int)\"), self.setNbExos)" % (6-level, box))
        for level in range(5):
            nb_exos = len(LESFICHES[level][2])
            for i in range(nb_exos):
                exec("self.label_%s_%s.setText(u\"%s\")" % (6-level, i,
                    LESFICHES[level][2][i]))
                exec("self.imglabel_%s_%s.setText(r'<img src=\"%s\" />')" %
                        (6-level, i, os.path.join(DATADIR, 'images',
                            'whatsthis.png')))
                exec("self.imglabel_%s_%s.setToolTip(r\'<img src=\"%s\" />\')" %
                        (6-level, i, os.path.join(DATADIR, 'images', 'vignettes',
                            '%se-%02d.png' % (6-level, i))))
                exec(u"self.spinBox_%s_%s.setToolTip(u\"Choisissez le nombre d\'exercices de ce type à créer.\")"% (6-level,i))

        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    #============================================================
    #        Début des fonctions
    #============================================================

    ### Gestion des erreurs
    def erreur_critique(self, message):
      """Dialogue si pyromaths.xml est défectueux."""
      reply = QtGui.QMessageBox.critical(self, "Erreur critique", message)
      if reply:
        sys.exit(1)


    def about(self):
        """Crée la boîte de dialogue "À propos de..." """
        text = u"""<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0//EN" "http://www.w3.org/TR/REC-html40/strict.dtd">
<html>
  <head>
    <meta name="qrichtext" content="1" />
    <style type="text/css">
      p, li { white-space: pre-wrap; align:"justify"; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:5px; -qt-block-indent:0; text-indent:0px; }
    </style>
  </head>
  <body style=" font-family:'DejaVu Sans'; font-size:9pt; font-weight:400; font-style:normal;" bgcolor="#f9efbe">
    <p align="center">
    <img src="%s" />
    <br /><br />
    <span style="font-weight:600;">Version %s</span>
    </p>
    <p>
    <span style=" font-weight:600;">Pyromaths</span> est un programme qui permet de créer des  fiches d'exercices types de mathématiques niveau collège avec leur corrigé.
    </p>
    <p>
    Les fiches sont produites au format LaTeX. Pyromaths lance ensuite les commandes nécessaires à la production de fichiers pdf (latex - dvips - ps2pdf) et les ouvre.
    </p>
    <p align="center">
    <span style=" text-decoration: underline;">Remerciements à&nbsp;:</span>
    </p>
    <p>
    <ul style="-qt-list-indent:1;">
      <li>
      <span style=" font-weight:600;">David Robert</span> pour l'idée de départ&nbsp;;
      </li>
      <li>
      <span style=" font-weight:600;">Yves Gesnel</span> pour le portage de Pyromaths sur Mac OS X&nbsp;;
      </li>
      <li>
      <span style=" font-weight:600;">Arnaud Kientz</span> pour ses graphismes, son implication dans le code de Pyromaths et son amitié&nbsp;;
      </li>
      <li>
      <span style=" font-weight:600;">Guillaume Barthélémy</span> pour ses exercices&nbsp;;
      </li>
      <li>
      <span style=" font-weight:600;">Nicolas Bissonnier</span> pour son arrivée dans l'équipe de développement&nbsp;;
      </li>
      <li>
      <span style=" font-weight:600;">Nicolas Pourcelot</span> pour ses conseils et son implication prochaine dans le code de Pyromaths&nbsp;;
      </li>
      <li>
      <span style=" font-weight:600;">Didier Roche</span> pour l'intégration de Pyromaths dans les dépôts Ubuntu&nbsp;;
      </li>
      <li>
      <span style=" font-weight:600;">Jacqueline Gouguenheim-Desloy</span> a porté Pyromaths sur Mac OS X à ses débuts. Son soutien et son amitié nous ont été précieux. Sa disparition est une perte douloureuse pour la communauté du logiciel libre.
      </li>
    </ul>
    </p>
    <p align="center">
    %s
    </p>
  </body>
</html>"""
        if sys.platform == "darwin":  #Cas de Mac OS X.
            banniere = os.path.join(DATADIR, 'images', 'pyromaths.png')
        else:
            banniere = os.path.join(DATADIR, 'images', 'pyromaths-banniere.png')
        QtGui.QMessageBox.about(None, u'À propos de Pyromaths', text % (banniere,  VERSION, COPYRIGHTS))

    def creer_les_exercices(self):
        """Vérifie si la liste d'exercices n'est pas vide puis sélectionne les noms des fichiers exercices et
        corrigés"""
        self.valide_options()
        if self.liste_creation == [] :
            QtGui.QMessageBox.warning(None, 'Attention !',
                    u"Veuillez sélectionner des exercices...",
                    QtGui.QMessageBox.Ok )
        else:
            parametres = {
                'creer_pdf': self.checkBox_pdf.isChecked(),
                'creer_unpdf': self.checkBox_unpdf.isChecked() and self.checkBox_unpdf.isEnabled(),
                'titre': unicode(self.titre_fiche.text()),
                'corrige': self.checkBox_corrige.isChecked(),
                'niveau': unicode(self.comboBox_niveau.currentText()),
                'nom_fichier': unicode(self.nom_fichier.text()),
                'chemin_fichier': unicode(self.chemin_fichier.text()),
                'modele': unicode(self.comboBox_modele.currentText() + '.tex'),
                'datadir': DATADIR,
                'configdir': CONFIGDIR
                         }
            #============================================================
            #        Choix de l'ordre des exercices
            #============================================================
            list=[]
            for i in range(len(self.liste_creation)):
                niveau = self.liste_creation[i][0]
                exo = self.liste_creation[i][1]
                list.append("%se: %s" % (6-niveau,  LESFICHES[niveau][2][exo]))
            self.List=QtGui.QListWidget()
            for i in range(len(list)):
                item = QtGui.QListWidgetItem(list[i])
                item.setFlags(QtCore.Qt.ItemIsEnabled |
                              QtCore.Qt.ItemIsSelectable |
                              QtCore.Qt.ItemIsDragEnabled)
                self.List.addItem(item)
            bmono=True
            for i in range(len(list)):
                if list[0] != list[i]: bmono=False
            if bmono:
                # S'il ny a qu'un seul type d'exercices, pas la peine de choisir
                # l'ordre
                valide(self.List, LESFICHES, parametres)
            else:
                form = ChoixOrdreExos(self.List,  LESFICHES,  parametres)
                form.exec_()

    def creer_tous_les_exercices(self):
        """
        Créer des fiches exemples pour tous les niveaux avec tous les exercices
        dans le dossier /home/jerome/workspace/Pyromaths/src/exemples
        """
        self.valide_options()
        d0 = ""
        d0 = unicode(QtGui.QFileDialog().getExistingDirectory (None, u"Dossier où créer les fiches",
                                                         self.config['chemin_fichier'], QtGui.QFileDialog.ShowDirsOnly))
        i = 0
        if d0:
            for niveau in range(5):
                liste = []
                for i in range(len(LESFICHES[niveau][2])):
                    liste.append((niveau,  i))
                if niveau != 4:
                  exo = os.path.join(d0, "%se.tex" % (6 - niveau))
                  cor = os.path.join(d0, "%se-corrige.tex" % (6 - niveau))
                  parametres = {
                                          'les_fiches': LESFICHES,
                                          'fiche_exo': exo,
                                          'fiche_cor': cor,
                                          'liste_exos': liste,
                                          'creer_pdf': '1',
                                          'titre': "Exemple de fiche",
                                          'niveau': "%s\\ieme" % (6-niveau),
                                          'modele': str(self.comboBox_modele.currentText() + '.tex'),
                                          'corrige': True,
                                          'creer_unpdf': True,
                                          'datadir': DATADIR,
                                          'configdir': CONFIGDIR
                                          }
                else:
                  exo = os.path.join(d0, "Lycee.tex" )
                  cor = os.path.join(d0, "Lycee-corrige.tex" )
                  parametres = {
                                          'les_fiches': LESFICHES,
                                          'fiche_exo': exo,
                                          'fiche_cor': cor,
                                          'liste_exos': liste,
                                          'creer_pdf': '1',
                                          'titre': "Exemple de fiche",
                                          'niveau': u"Lycée",
                                          'modele': str(self.comboBox_modele.currentText() + '.tex'),
                                          'corrige': True,
                                          'creer_unpdf': True,
                                          'datadir': DATADIR,
                                          'configdir': CONFIGDIR
                                          }
                System.creation(parametres)

    def effacer_choix_exercices(self):
        """Remet toutes les SpinBox à zéro et vide la liste d'exercices sélectionnés"""
        self.liste_creation=[]
        for level in range(5):
            for box in range(len(LESFICHES[level][2])):
                exec("self.spinBox_%s_%s.setValue(0)" % (6-level, box))

    def enregistrer_config(self):
        """Fonction qui se charge d'enregistrer les options de l'interface dans le fichier de configuration
        après avoir complété le dictionnaire."""
        tree = lxml.etree.parse(self.configfile)
        root = tree.getroot()
        options = root.find('options')
        options .find('nom_fichier').text = unicode(self.nom_fichier.text())
        options .find('chemin_fichier').text = unicode(self.chemin_fichier.text())
        options .find('titre_fiche').text = unicode(self.titre_fiche.text())
        options .find('corrige').text = str(self.checkBox_corrige.isChecked())
        options .find('pdf').text  = str(self.checkBox_pdf.isChecked())
        options .find('unpdf').text  = str(self.checkBox_unpdf.isChecked())
        options .find('modele').text = unicode(self.comboBox_modele.currentText() + '.tex')

        f = codecs.open(self.configfile, encoding='utf-8', mode='w')
        f.write(lxml.etree.tostring(root, pretty_print=True, encoding="UTF-8",
                               xml_declaration=True).decode('utf-8', 'strict'))
        f.close()

    def insert_spinbox(self, level, box):
        """Place autant de SpinBox que d'exercices pour chaque niveau et les nomme"""
        treated="%s_%s" % (level, box)
        exec("self.horizontalLayout_%s = QtGui.QHBoxLayout()" % (treated))
        exec("self.spinBox_%s = QtGui.QSpinBox(self.tab_%se)" % (treated, level))
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(40)
        sizePolicy.setVerticalStretch(30)
        exec("sizePolicy.setHeightForWidth(self.spinBox_%s.sizePolicy().hasHeightForWidth())" % (treated))
        exec("self.spinBox_%s.setSizePolicy(sizePolicy)" % (treated))
        exec("self.horizontalLayout_%s.addWidget(self.spinBox_%s)" % (treated, treated))
        exec("self.imglabel_%s = QtGui.QLabel(self.tab_%se)" % (treated, level))
        exec("self.horizontalLayout_%s.addWidget(self.imglabel_%s)" % (treated, treated))
        exec("self.label_%s = QtGui.QLabel(self.tab_%se)" % (treated, level))
        exec("self.horizontalLayout_%s.addWidget(self.label_%s)" % (treated, treated))

        exec("spacerItem_%s = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)" % (treated))
        exec("self.horizontalLayout_%s.addItem(spacerItem_%s)" % (treated, treated))
        exec("self.horizontalLayout_%s.addItem(spacerItem_%s)" % (treated, treated))

        exec("self.gridLayout_%se.addLayout(self.horizontalLayout_%s, %s, %s, 1, 1)" % (level, treated, box/2, box%2))

    def lire_config(self,  section):
        """Lis le fichier de configuration pyromaths.conf, enregistre les données dans un dictionnaire config"""
        config = {}
        tree = lxml.etree.parse(self.configfile)
        root = tree.getroot()
        options = root.find(section)
        for child in options:
            if child.text == 'True': text = '1'
            elif child.text == 'False': text = '0'
            else : text = child.text
            config[child.tag] = text
        return config

    def option_parcourir(self):
        d0 = QtGui.QFileDialog().getExistingDirectory (None, u"Dossier où créer les fiches",
                                                       self.config['chemin_fichier'], QtGui.QFileDialog.ShowDirsOnly)
        if d0:
            self.chemin_fichier.setText(d0)

    def option_corrige(self):
        if not self.checkBox_corrige.isChecked():
            self.checkBox_unpdf.setChecked(False)
            self.checkBox_unpdf.setEnabled(False)
        else:
            self.checkBox_unpdf.setEnabled(True)

    def setNbExos(self):
        """Modifie le nombre d'exercices dans la variable liste_creation lorsqu'on  modifie une spinBox
        et adapte le niveau affiché dans l'en-tête de la fiche en fonction du plus haut niveau d'exercice"""
        niveau=0
        self.liste_creation = []
        for level in range(5):
            for box in range(len(LESFICHES[level][2])):
                exec("qte = self.spinBox_%s_%s.value()" % (6 - level, box),  locals(),  globals())
                for i in range(qte):
                    exec("self.liste_creation.append((%s, %s))" % (level, box))
                    if level > niveau:
                        niveau = level
        self.comboBox_niveau.setCurrentIndex(niveau)

    def site(self):
        """Ouvre le navigatuer internet par défaut sur la page d'accueil du site http://www.pyromaths.org"""
        import webbrowser
        webbrowser.open('http://www.pyromaths.org')


    def valide_options(self):
        """Synchronise les options éventuellement saisies par l'utilisag
QCoreApplication::exec: The event loop is already runningteur avec le dictionnaire de config"""
        self.config['chemin_fichier'] = self.chemin_fichier.text()
        self.config['nom_fichier'] = self.nom_fichier.text()
        self.config['titre_fiche'] = self.titre_fiche.text()
        self.config['corrige'] = self.checkBox_corrige.isChecked()
        self.config['pdf'] = self.checkBox_pdf.isChecked()
        self.config['unpdf'] = self.checkBox_unpdf.isChecked()and self.checkBox_unpdf.isEnabled()

#================================================================
#        Classe ChoixOrdreExos
#================================================================

class ChoixOrdreExos(QtGui.QDialog):
    """À appeler de la façon suivante :
    form = ChoixOrdreExos(list, LesFiches, parametres)
    Permet de choisir l'ordre dans lequel les exercices vont apparaître
    parametres = {'fiche_exo':
                  'fiche_cor':
                  'liste_exos':
                  'creer_pdf':
                  'titre':
                  'corrige':
                  'niveau':
                  'nom_fichier':
                  'chemin_fichier':
                 }"""

    def __init__(self, list, LesFiches, parametres,  parent=None):
        LESFICHES = LesFiches
        self.parametres = parametres
        self.List = list
        QtGui.QDialog.__init__(self, parent)
        self.setWindowTitle("Choisissez l'ordre des exercices")
        layout = QtGui.QHBoxLayout()

        buttonBox = QtGui.QDialogButtonBox()
        buttonBox.setOrientation(QtCore.Qt.Vertical)
        buttonBox.setStandardButtons(QtGui.QDialogButtonBox.Cancel|QtGui.QDialogButtonBox.Ok)

        self.List.setAlternatingRowColors(True)
        self.List.setDragEnabled(True)
        self.List.setAcceptDrops(True)
        self.List.setDragDropMode(QtGui.QAbstractItemView.InternalMove)
        self.List.setDropIndicatorShown(True)

        layout.addWidget(self.List)
        layout.addWidget(buttonBox)
        self.setLayout(layout)

        QtCore.QObject.connect(buttonBox, QtCore.SIGNAL("accepted()"), self.accept)
        QtCore.QObject.connect(buttonBox, QtCore.SIGNAL("rejected()"), self.close)

    def accept(self):
        """Écrit une liste contenant la liste des exercices dans l'ordre choisit par l'utilisateur et demande à
        celui-ci les noms de fichiers pour les exercices et les corrigés"""
        valide(self.List, LESFICHES, self.parametres)
        self.close()

def valide(list, LesFiches, parametres):
    """ Permet de choisir les noms et emplacements des fichiers tex, les écrits
    et lance la compilation LaTex"""
    corrige = parametres['corrige']
    l=[]
    lesexos = []
    for i in range(list.count()):
        l.append(unicode(list.item(i).text()))
    for text in l:
        niveau = 6 - int(text[0])
        pos = LesFiches[niveau][2].index(text[4:])
        lesexos.append((niveau,  pos))

    #============================================================
    #        Choix des noms des fichiers exercices et corrigés
    #============================================================
    (f0, f1) = ("", "")
    saveas = QtGui.QFileDialog()
    filename = System.supprime_extension(parametres['nom_fichier'],
                                         '.tex')
    f0 = unicode(saveas.getSaveFileName(None, "Enregistrer sous...",
                os.path.join(parametres['chemin_fichier'],
                             u'%s.tex' % filename), "Documents Tex (*.tex)"))
    if f0 != None:
        System.ajoute_extension(f0, '.tex')
        if corrige and not parametres['creer_unpdf']:
            f1 = unicode(saveas.getSaveFileName(None, "Enregistrer sous...",
                os.path.join(os.path.dirname(f0),
                u"%s-corrige.tex"  % os.path.splitext(os.path.basename(f0))[0]),
                "Documents Tex (*.tex)"))
        else:
            f1 = os.path.join(os.path.dirname(f0), u"%s-corrige.tex"  %
                              os.path.splitext(os.path.basename(f0))[0])
        if f1 != None:
            if corrige:
                System.ajoute_extension(f1, '.tex')
            parametres ['fiche_exo'] = f0
            parametres ['fiche_cor'] = f1
            parametres ['liste_exos'] = lesexos
            parametres ['les_fiches'] =  LesFiches
            System.creation(parametres)



# -*- coding: utf-8 -*-

import re, sys
from Affichage import decimaux
from . import Arithmetique, Affichage
import math
from .Fractions import EffectueSommeFractions, EffectueQuotientFractions, EffectueProduitFractions

def EffectueCalcul(op, nb1, nb2, pre="", post=""):
    denominateur_max = 20
    programme = 6  # Faisable pour toutes les classes
    fr = Fractions(1, 1)  # Une fraction type pour vérifier le type de nb1 et nb2
    bfr = type(nb1) == type(nb2) == type(fr)  # bfr est vrai si on calcul avec des fractions
    if type(nb1) == type(fr) and type(nb2) != type(fr):
        nb2 = Fractions(nb2)
        bfr = True
    if type(nb2) == type(fr) and type(nb1) != type(fr):
        nb1 = Fractions(nb1)
        bfr = True
    if not bfr and pre.rstrip().endswith('(') and post.lstrip().startswith(')'):
        pre = pre.rstrip()[:len(pre.rstrip()) - 1]
        post = post.lstrip()[1:]
    if op == "+":
        if bfr:
            l = EffectueSommeFractions(nb1, nb2, op, pre, post)
            resultat = Fractions.simplifie(nb1 + nb2)
            if abs(resultat.d) > denominateur_max:
                programme = 2
        else:
            l = ["%s%s%s" % (pre, Affichage.decimaux(nb1 + nb2, 1), post)]
            resultat = nb1 + nb2
    elif op == "-":
        if bfr:
            l = EffectueSommeFractions(nb1, nb2, op, pre, post)
            resultat = Fractions.simplifie(nb1 - nb2)
            n = nb1 - nb2
            if n.n < 0:
                programme = 4
            if resultat.n * resultat.d < 0:
                programme = 4
            if abs(resultat.d) > denominateur_max:
                programme = 2
        else:
            l = ["%s%s%s" % (pre, Affichage.decimaux(nb1 - nb2, 1), post)]
            resultat = nb1 - nb2
            if resultat < 0:
                programme = 4
    elif op == "*":
        if bfr:
            l = EffectueProduitFractions(nb1, nb2, pre, post)
            resultat = Fractions.simplifie(nb1 * nb2)
            if abs(resultat.d) > denominateur_max:
                programme = 2
        else:
            l = ["%s%s%s" % (pre, Affichage.decimaux(nb1 * nb2, 1), post)]
            resultat = nb1 * nb2
    else:
        if bfr:
            if nb2.n:  # Pas de division par zéro
                l = EffectueQuotientFractions(nb1, nb2, pre, post)
                resultat = Fractions.simplifie(nb1 / nb2)
                if abs(resultat.d) > denominateur_max:
                    programme = 2
            else:
                (l, resultat, programme) = ([], 1, 1)
        else:
            if nb2 == 0:  # division par zéro
                (l, resultat, programme) = ([], 1, 1)
            else:
                l = ["%s%s%s" % (pre, Affichage.decimaux((nb1 * 1.0) /
                     nb2, 1), post)]
                resultat = (nb1 * 1.0) / nb2
                if resultat - math.floor(resultat) == 0.0:
                    programme = 6
                elif resultat * 100 - math.floor(resultat * 100) == 0.0:
                    programme = 5
                else:
                    programme = 2
    return (l, resultat, programme)


def split_calc(calcul):

    """Test de gestion et affichage des priorités de calcul, détaillées. PAS
    FINI."""

    liste_op = []
    liste_result = []

    init = calcul  # Sauvegarde du calcul de départ

    # Construction d'une liste comportant la liste de toutes les opérations
    i = 0

    while calcul.count('(') > 0:
        current = 'l' + str(i)
        liste_op.append(re.search('\(\w+[\+\-/\*]\w+\)',
            calcul).group().strip('()'))
        calcul = re.sub('\(\w+[\+\-/\*]\w+\)', current, calcul, 1)
        i += 1

    liste_op.append(re.search('\w+[\+\-/\*]\w+', calcul).group())

    # Calcul des différentes opérations contenues dans liste_op et
    # enregistrement des résultats dans liste_result

    for index in range(len(liste_op)):
        element = liste_op[index]
        m = re.findall('l(\d+)', element)

        for k in range(len(m)):
            motif = 'liste_result[' + m[k] + ']'
            element = re.sub('l\d+', motif , element, 1)

        liste_op[index] = element

        liste_result.append(eval(liste_op[index]))

    print liste_op
    print liste_result, liste_result[0]

    # On sert de la sauvegarde pour afficher le déroulement du calcul
    j = 0

    print init, " = ",
    longueur = len(init) + 2

    while init.count('(') > 0:
        current = 'l' + str(i)
        init = re.sub('\(\w+[\+\-/\*]\w+\)', str(eval(liste_op[j])), init, 1)
        j += 1

        print(init + "\n" + " " * longueur + "= ",)

    # Afficher le résultat du calcul
    print(liste_result[-1])



#a=testfraction.fraction(5,7)
#b=testfraction.fraction(4,3)
#c=testfraction.fraction(8,9)
#
#
#c=split_calc('(a+b)*c')


#calcul = raw_input("Un autre calcul ?\nIl faut mettre des parenthèses à chaque opération, même si inutiles.\n")

#split_calc(calcul)

#TODO : étendre à des nombres avec plusieurs chiffres
#---------------------------------------------------------------------
#autre version des priorités
#---------------------------------------------------------------------

import re

def priorites(calcul, pre = "", post = "", solution = []):
    if not solution: solution = [calcul]
    recherche_parentheses = r"""
    ^(.*)                           #des calculs dans le groupe 0
    (                               #début du groupe 1
      \(                            #commence par une parenthèse ouvrante
      (?:                           #regroupement 0 (non groupe)
        [\+\-]?                     #un éventuel signe
        (?:                         #regroupement 1 (non groupe)
          \(                        #une parenthèse ouvrante
          [\+\-]?                   #un éventuel signe
          \d+\.?\d*(?:e[\+\-]\d+)?  #on cherche un nombre avec une éventuelle
                                    #partie décimale et un éventuel exposant
          \)                        #une parenthèse fermante
        |                           #ou
          \d+\.?\d*(?:e[\+\-]\d+)?  #on cherche un nombre avec une éventuelle
                                    #partie décimale et un éventuel exposant
        |                           #ou
          \w+                       #un littéral
        )                           #fin du regroupement 1
        [\+\-\*/]+                  #un opérateur
      )+                            #fin du regroupement 0 (présent au moins
                                    #un fois)
      [\+\-]?                     #un éventuel signe
      (?:                           #regroupement 0 (non groupe)
          \(                        #une parenthèse ouvrante
          [\+\-]?                   #un éventuel signe
          \d+\.?\d*(?:e[\+\-]\d+)?  #on cherche un nombre avec une éventuelle
                                    #partie décimale et un éventuel exposant
          \)                        #une parenthèse fermante
        |                           #ou
          \d+\.?\d*(?:e[\+\-]\d+)?  #on cherche un nombre avec une éventuelle
                                    #partie décimale et un éventuel exposant
        |                           #ou
        \w+                         #un littéral
      )                             #fin du regroupement 0 présent une seule fois
      \)                            #fini par une parenthèse fermante
    )                               #fin du groupe 1
    (.*)$                           #des calculs pour finir
    """
    test = re.search(recherche_parentheses, calcul, re.VERBOSE)
    while test:
        sous_calcul = test.groups()
        (calcul, solution) = priorites_operations(sous_calcul[1],
                                                  sous_calcul[0],
                                                  sous_calcul[2],
                                                  solution)
        calcul = solution[-1]
        test = re.search(recherche_parentheses, calcul, re.VERBOSE)
    calcul, solution = priorites_operations(calcul, pre, post, solution)
    return(solution)

def traite_operation(operateur, calcul, pre, post, solution):
    pattern = r"""
            (                   #groupe 0
                ^\(*                #le calcul est au début (aux parenthèses près)
                                    #les signes +/- font parti du groupe 2
            |                   #ou
                ^.*?[\+\-]?         #il y a des calculs avant, le 1er signe +/-
                                    #est un opérateur
            )                   #fin du groupe 0
            (                   #groupe 1
                (?:                 #regroupement 1 (non groupe)
                    [\+\-]?                 #un éventuel signe
                    \(                      #une parenthèse ouvrante
                    [\+\-]?                 #un éventuel signe
                    \d+\.?\d*(?:e[\+\-]\d+)? #on cherche un nombre avec une
                                            #éventuelle partie décimale et un
                                            #éventuel exposant
                    \)                     #une parenthèse fermante
                |                   #ou
                    [\+\-]?                 #un éventuel signe
                    \d+\.?\d*(?:e[\+\-]\d+)? #on cherche un nombre avec une
                                            #éventuelle partie décimale et un
                                            #éventuel exposant
                |                   #ou
                    \w+                     #un littéral
                )                   #fin du regroupement 1
                {0}                 #opération
                (?:                 #regroupement 2 (non groupe)
                    [\+\-]?                 #un éventuel signe
                    \(                     #une parenthèse ouvrante
                    [\+\-]?                 #un éventuel signe
                    \d+\.?\d*(?:e[\+\-]\d+)? #on cherche un nombre avec une
                                            #éventuelle partie décimale et un
                                            #éventuel exposant
                    \)                     #une parenthèse fermante
                |                   #ou
                    [\+\-]?                 #un éventuel signe
                    \d+\.?\d*(?:e[\+\-]\d+)? #on cherche un nombre avec une
                                            #éventuelle partie décimale et un
                                            #éventuel exposant
                |                           #ou
                    \w+                     #un littéral
                )                   #fin du regroupement 2
            )                   #fin du groupe 1
            (.*?)$              #tout le reste
            """.format(operateur)
    test = re.search(pattern, calcul, re.VERBOSE)
    while test:
        sous_calcul = test.groups()
        print(sous_calcul)
        print(pre, post)
        sous_resultat = eval(sous_calcul[1])
        if isinstance(sous_resultat, list):
            for element in sous_resultat:
                solution.extend([pre +
                             sous_calcul[0] +
                             str(element) +
                             sous_calcul[2] +
                             post])
            a = sous_resultat[-1]
            sous_resultat = 'a'
        elif (sous_calcul[0] and sous_calcul[0][-1] == '(') and \
           (sous_calcul[2][0] and sous_calcul[2][0] == ')') and \
           (post[:2] != '**' or sous_resultat > 0):
            #permet de supprimer les parenthèses
            solution.extend([pre +
                             sous_calcul[0][:-1] +
                             decimaux(sous_resultat, True) +
                             sous_calcul[2][1:] +
                             post])
        else:
            solution.extend([pre +
                             sous_calcul[0] +
                             decimaux(sous_resultat, True) +
                             sous_calcul[2] +
                             post])
        calcul = sous_calcul[0] + decimaux(sous_resultat, True) + sous_calcul[2]
        test = re.search(pattern, calcul, re.VERBOSE)
    return (calcul, pre, post, solution)

def priorites_operations(calcul, pre = "", post = "", solution = []):
    for string in ["\*\*", "[\*/]", "[\+\-]"]:
        calcul, pre, post, solution = traite_operation(string, calcul, pre,
                                                   post, solution)
    #calcul, pre, post, solution = traite_operation("\*\*", calcul, pre,
                                                   #post, solution)
    #calcul, pre, post, solution = traite_operation("[\*/]", calcul, pre,
                                                   #post, solution)
    #calcul, pre, post, solution = traite_operation("[\+\-]", calcul, pre,
                                                   #post, solution)
    return (calcul, solution)

##
##print(priorites("(12-15)**2+(15-12)**3"))
##print("_" * 40)
###print(priorites("((-(-1)+-2)++3)"))
###print("_" * 40)
###print(priorites("4-((-1+-2)+3)"))
###print("_" * 40)
###print(priorites('8-(-5*(2-4))'))
###print("_" * 40)
##print(priorites("1+2+3+4"))
##print("_" * 40)
###print(priorites("-3**+2"))
###print("_" * 40)
###print(priorites("9+8*7"))
###print("_" * 40)
###print(priorites("8+1.3e+2"))
###print("_" * 40)
##a, b, c = Racine(5), Racine(5, 3), Racine(5, 2)
##print(priorites("2*(a+b+c)"))
##print("_" * 40)
##a, b = Racine(5), Racine(5, 3)
##print(priorites("a*b*2"))
###print("_" * 40)
###c, d = Terme(5, 2), Terme(-6, 2)
###print(priorites("c*d"))
###print("_" * 40)
##print(priorites("8+1.3e+2"))
##print(priorites("755*(12-15)**2+(15-12)**3"))
##print(priorites("((-(-1)+-2)++3)"))
##print(priorites("4-((-1+-2)+3)"))
##print(priorites('8-(-5*(2-4))'))
##print(priorites("1+2+3+4"))
##print(priorites("-3**+2"))
##print(priorites("9+8*7"))

from ..classes.Fractions import Fractions
from . import Arithmetique, Affichage
import random
import math
from TeXMiseEnForme import Affichage

#===============================================================================
# Gère les priorités opératoires pour le calcul décimal et fractionnaire
#===============================================================================


def OperateurPrioritaire(exercice, niveau, pre="", post="", solution=[]):
    '''Renvoie un tuple contenant :
    - la liste solution contenant la liste des instructions TeX \xc3\xa0 \xc3\xa9crire dans le corrig\xc3\xa9
    - le r\xc3\xa9sultat de l\'expression.
    La liste exercice peut contenir des nombres ou des objets Fractions.
    Ex : OperateurPrioritaire([3,"+",5,"*",2]) => [\'3 + 10\', \'13\'] , 13
    Ex : OperateurPrioritaire([Fractions(15,4),\'*\',Fractions(14,25),\'-\',Fractions(3,2)])
         => [ \'\\dfrac{3 \\times \\cancel{5}}{2 \\times \\bcancel{2}} \\times \\dfrac{7 \\times \\bcancel{2}}{5 \\times \\cancel{5}} - \\dfrac{3}{2}\',
              \'\\dfrac{21}{10} - \\dfrac{3}{2}\',
              \'\\dfrac{21}{10}-\\dfrac{3_{\\times 5}}{2_{\\times 5}}\',
              \'\\dfrac{3_{\\times 2}}{5_{\\times 2}}\',
              \'\\dfrac{3}{5}\'
            ],
            <pyro_classes.Fractions instance at 0xb7ddddec> (Fractions(3,5))

    @param exercice: liste contenant la succession d\'op\xc3\xa9rations [3, \'+\', 5, \'*\', 2]
    @param niveau: niveau auquel doit se situer l\'exercice : 6: entiers, 5: d\xc3\xa9cimaux positifs, 4: d\xc3\xa9cimaux relatifs
    @param pre: chaine contenant ce qui doit \xc3\xaatre ajouter avant la solution
    @param post: chaine contenant ce qui doit \xc3\xaatre ajouter apr\xc3\xa8s la solution
    @param solution: liste contenant chaque ligne \xc3\xa0 \xc3\xa9crire dans le fichier TeX pour
                     le corrig\xc3\xa9
    '''

    if len(exercice) == 1:  # il ne reste plus qu'un nombre (le résultat) donc c'est fini
        return (solution, exercice[0], niveau)
    else:

          # Trouvons l'opérateur prioritaire :

        nbpar = exercice.count('(')
        nbmul = exercice.count("*")
        nbdiv = exercice.count("/")
        if nbpar:  # Des parenthèses oui, mais y-a-t-il des parenthèses à l'intérieur des parenthèses ?
            i0 = TrouveParentheseInterieure(exercice)
            i1 = exercice[i0:].index(')') + i0
            pre1 = Affichage(exercice[:i0 + 1])
            post1 = Affichage(exercice[i1:])
            (cor, res, programme) = OperateurPrioritaire(exercice[i0 + 1:i1],
                    niveau, pre + pre1, post + post1, solution)
            suite_exo = exercice[:i0]
            suite_exo.append(res)
            suite_exo.extend(exercice[i1 + 1:])
            return OperateurPrioritaire(suite_exo, programme, pre, post,
                    solution)
        else:
            if nbmul or nbdiv:

                # Priorité à la multiplication et à la division

                if nbmul and not nbdiv:
                    i0 = exercice.index("*")
                elif nbdiv and not nbmul:
                    i0 = exercice.index("/")
                elif exercice.index("*") < exercice.index("/"):
                    i0 = exercice.index("*")
                else:
                    i0 = exercice.index("/")
            else:

                # Il ne reste que des additions et des soustractions

                nbadd = exercice.count("+")
                nbsub = exercice.count("-")
                if nbadd and not nbsub:
                    i0 = exercice.index("+")
                elif nbsub and not nbadd:
                    i0 = exercice.index("-")
                elif exercice.index("+") < exercice.index("-"):
                    i0 = exercice.index("+")
                else:
                    i0 = exercice.index("-")
            pre1 = Affichage(exercice[:i0 - 1])  # Ce qui est avant le calcul en cours
            post1 = Affichage(exercice[i0 + 2:])  # Ce qui est après le calcul en cours
            (l, res, programme) = EffectueCalcul(exercice[i0], exercice[i0 -
                    1], exercice[i0 + 1], pre + pre1, post + post1)
            for i in l:
                solution.append(i)
            suite_exo = exercice[:i0 - 1]
            suite_exo.append(res)
            suite_exo.extend(exercice[i0 + 2:])
            programme = min(programme, niveau)
            return OperateurPrioritaire(suite_exo, programme, pre, post,
                    solution)



def TrouveParentheseInterieure(l):
    """Trouve la position de la parenth\xc3\xa8se la plus \xc3\xa0 l'int\xc3\xa9rieur

    @param l: liste d'op\xc3\xa9rations
    """

    nbpar = l.count('(')
    if nbpar > 1:
        if l[l.index('(') + 1:].index('(') < l[l.index('(') + 1:].index(')'):  # Il existe une parenthèse dans les parenthèses
            return l.index('(') + 1 + TrouveParentheseInterieure(l[l.index('(') +
                    1:])
        else:
            return l.index('(')
    else:
        return l.index('(')


def valeurs_priorites_decimaux(nb, entier=1):  # renvoie les 2 listes contenant les opérateurs et les opérandes.
    listoperateurs = [
        "+",
        "*",
        "-",
        "/",
        '(',
        '(',
        '(',
        '(',
        ')',
        ')',
        ')',
        ')',
        ]
    loperateurs = []
    loperandes = []
    i = 0  #nombre d'opérateurs créés
    p = 0  #nombre de parenthèses ouvertes
    cpt = 0  #compteur pour éviter que le programme ne boucle.
    while i < nb - 1:
        cpt = cpt + 1
        if cpt > 10:  #On recommence
            (cpt, i, p, loperateurs) = (0, 0, 0, [])
        if p:
            if loperateurs[-1] == '(':  # On n'écrit pas 2 parenthèses à suivre
                operateur = listoperateurs[random.randrange(4)]
            else:
                operateur = listoperateurs[random.randrange(12)]
        elif loperateurs == []:

            # On ne commence pas par une parenthèse

            operateur = listoperateurs[random.randrange(4)]
        else:
            operateur = listoperateurs[random.randrange(8)]
        if nb > 3:
            test = ('-*/').find(operateur) >= 0 and loperateurs.count(operateur) < \
                1 or operateur == "+" and loperateurs.count(operateur) < \
                2
        else:
            test = ('-*/+').find(operateur) >= 0 and loperateurs.count(operateur) < \
                1
        if test:

            #On n'accepte pas plus de 1 produit, différence, quotient et de 2 sommes ou parenthèses par calcul.

            if i == 0 or loperateurs[-1] != '(' or ('*/').find(operateur) < \
                0:  #pas de * ou / dans une parenthèse.
                i = i + 1
                loperateurs.append(operateur)
        elif operateur == '(' and (')+').find(loperateurs[-1]) < 0:

            #Il ne peut y avoir de ( après une ) ou après un +

            p = p + 1
            loperateurs.append(operateur)
        elif operateur == ')':
            p = p - 1
            loperateurs.append(operateur)
    while p > 0:
        loperateurs.append(')')
        p = p - 1
    if entier:
        loperandes = [random.randrange(12) + 2 for i in range(nb)]
    else:
        loperandes = [((random.randrange(88) + 12) * 1.0) / 10 for i in
                      range(nb)]
    exercice = [loperandes[0]]
    i = 1
    j = 0
    while i < len(loperandes) or j < len(loperateurs):
        if j < len(loperateurs):
            exercice.append(loperateurs[j])
            j = j + 1
        while j < len(loperateurs) and (loperateurs[j] == '(' or
                loperateurs[j - 1] == ')'):
            exercice.append(loperateurs[j])
            j = j + 1
        if i < len(loperandes):
            exercice.append(loperandes[i])
            i = i + 1
    return exercice


#for i in xrange(1):
#    l=valeurs_priorites_fractions(3, 1)
#    cor,res, programme=OperateurPrioritaire(l,6,pre="", post="",solution=[])
#    while programme<5:
#        l=valeurs_priorites_fractions(3, 1)
#        cor,res,programme=OperateurPrioritaire(l,6,pre="", post="",solution=[])
#    print "$" + Affichage(l)+"$\\par"
#    for i in cor:
#        print "$" + i +"$\\par"







#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#
#----------------------------------------------------------------------
# Pyromaths : GESTION DES PRIORITES
#----------------------------------------------------------------------
from ..classes.PolynomesCollege import Polynome
from ..outils.Affichage import decimaux


def cherche_polynome(calcul, index):
    """Recherche le premier polynôme dans la chaîne calcul à une position
    supérieure ou égale à index, sans expression régulière
    @calcul: str
    @index: int"""
    index = calcul.find("Polynome(", index)
    if index < 0: return None
    par, i = 1, index+9 # nombre de parenthèses ouvertes, début de recherche
    while par:
        if calcul.find("(", i)>0:
            i = min(calcul.find("(", i), calcul.find(")", i))
        else:
            i = calcul.find(")", i)
        if calcul[i] == ")": par += -1
        else: par += 1
    return calcul[index:i+1]

def cherche_decimal(calcul, index):
    """130 fois plus rapide qu'avec une expression régulière !
    Un nombre décimal peut être entouré de parenthèses s'il a un signe, ou avoir
    un signe s'il est en début de chaîne
    @calcul: str
    @index: int"""
    end = len(calcul)
    while index<=end:
        try:
            int(calcul[index])
        except:
            index+=1
        else:
            break
    if index>end: return None
    while end>index:
        try:
            float(calcul[index:end])
        except ValueError:
            end -= 1
        else:
            break
    if index>1 and (calcul[index-2:index] == "(+" or calcul[index-2:index] ==
            "(-") :
        if end < len(calcul) and calcul[end]==")":
            index -= 2
            end += 1
        else:
            index -= 1
    return calcul[index:end]

def split_calcul(calcul):
    """Partitionne la chaîne de caractères pour obtenir une liste d'opérateurs,
    de parenthèses, de polynômes et de nombres
    @calcul: str"""
    l = []
    findings = (cherche_polynome, cherche_decimal)
    for finding in findings:
        nb = finding(calcul, 0)
        while nb:
            l.extend(calcul.partition(nb))
            if l[-3] and l[-2]:
                l0 = split_calcul(l[-3])
                if l0:
                    l0.extend(l[-2:])
                    l = l[:-3]
                    l.extend(l0)
            elif not l[-3]:
                l.pop(-3)
            calcul = l[-1]
            nb = finding(calcul, 0)
            l = l[:-1]
    if calcul: l.append(calcul)
    while l and not l[0]:
        l.pop(0)
    return l

def EstNombre(value):
    """Test si Value est un nombre ou pas
    @value: str"""
    if value in "**/+-()":
        return False
    else:
        return isinstance(eval(value), (float, int, Polynome))

def splitting(calcul):
    """Partitionne la chaîne de caractères pour obtenir une liste d'opérateurs,
    de parenthèses, de polynômes et de nombres puis arrange la liste des
    opérandes et opérateurs
    @calcul: str"""
    if calcul == "": return []
    l = split_calcul(calcul)
    if l[0] == "+" and len(l) > 1: l[0] += l.pop(1)
    elif l[0] == "-" and len(l) > 1 and l[1][0] in "0123456789": l[0] += l.pop(1)
    j = 0
    while j < len(l):
        op, i, t = l[j], 0, []
        if op == "**": j+= 1
        else:
            while i < len(op) and op[i] in '+-*/()':
                t.append(op[i])
                i += 1
            if i == len(op):
                l[j:j+1] = t
                j += len(t)
            else: j += 1
    return l

def recherche_parentheses(calcul):
    """Recherche les parenthèses intérieures dans une liste
    @calcul: list"""
    if calcul.count("("):
        debut = calcul.index("(")
        if calcul[debut+1:].count(")"): fin = calcul[debut:].index(")") + debut
        else: return None
        for i in range(calcul[debut + 1:fin].count("(")):
            debut += calcul[debut+1:].index("(") + 1
        return (debut, fin+1)
    else:
        return None

def recherche_puissance(calcul):
    """Recherche la première occurrence d'une puissance dans une liste de
    calculs. Elle doit être entourée de deux nombres.
    @calcul: list"""
    if calcul.count("**"):
        i = calcul.index("**")
        return (i-1, i+2)
    else:
        return None

def recherche_operation(calcul, op, l_interdit, debut = 0):
    """Recherche la première occurrence de l'opération op dans la liste de
    calculs list. L'opérateur doit être entouré de deux nombres, le premier ne
    doit pas être précédé d'un opérateur inclus dans la liste l_interdit.
    L'opérateur à trouver doit être avant la position debut (si debut est non
    nul).
    @calcul: list
    @op: str
    @l_interdit: list
    @debut: int"""
    ind, fin = 0, len(calcul)
    for c in range(calcul.count(op)):
        ind = calcul.index(op, ind)
        if ind == 1 and ind < fin-1 and EstNombre(calcul[ind-1]) and\
            EstNombre(calcul[ind+1]):
                if debut: debut = min(debut, ind)
                else: debut = ind
                break
        elif ind > 1 and ind < fin -1 and EstNombre(calcul[ind-1]) and\
            EstNombre(calcul[ind+1]) and calcul[ind-2] not in l_interdit:
                if debut: debut = min(debut, ind)
                else: debut = ind
                break
        else:
            ind += 1
    if debut: return debut
    else: return None

def recherche_fin_operation(calcul, op, l_interdit, debut):
    """Recherche la fin de la suite d'opération @op situé à la position @debut
    de la liste de calculs @calcul, en vérifiant que cet enchaînement n'est pas
    suivi par un opérateur compris dans @l_interdit.
    @calcul: list
    @op: str
    @l_interdit: list
    @debut: int"""
    index, fin, unique = debut + 2, len(calcul), False
    while index < fin-1 and calcul[index] == op and EstNombre(calcul[index+1]):
        index +=2
    if index < fin and calcul[index] in l_interdit:
        index -=2
    if index == debut: return None
    return index

def recherche_produit(calcul):
    """Recherche la première occurrence d'une multiplication ou divisions dans
    une liste de calculs. Elle doit être précédée d'un nombre qui ne doit pas
    être précédé d'une puissance ou d'une division, NI SUIVIE D'UN EXPOSANT !
    @calcul: list"""
    debut = recherche_operation(calcul, "*", ("**", "/"))
    debut = recherche_operation(calcul, "/", ("**", "/"), debut)
    if debut is not None:
        fin = recherche_fin_operation(calcul, calcul[debut], ("**"), debut)
        if fin is not None: return (debut - 1, fin)
    return None

def recherche_somme(calcul):
    """Recherche la première occurrence d'une addition ou soustraction dans une
    liste de calculs. Elle doit être précédée d'un nombre qui ne doit pas être
    précédé d'une puissance, d'une multiplication, d'une division ou d'une
    soustraction, NI SUIVIE D'UN EXPOSANT, D'UNE MULTIPLICATION OU D'UNE
    DIVISION !
    @calcul: list"""
    debut = recherche_operation(calcul, "+", ("**", "*", "/", "-"))
    debut = recherche_operation(calcul, "-", ("**", "*", "/", "-"), debut)
    if debut is not None:
        fin = recherche_fin_operation(calcul, calcul[debut], ("**", "*", "/"),
                debut)
        if fin is not None: return (debut - 1, fin)
    return None

def recherche_neg(calcul):
    """Recherche la première occurence d'un opposé ## -(-5) ou -Polynome ## ou
    d'une écriture du style +(-5) dans une liste de calculs ET NE DOIT PAS ĘTRE
    SUIVI D'UN EXPOSANT !
    @calcul: list"""
    ind, debut, fin = 0, None, len(calcul)
    for c in range(calcul.count("-")):
        ind = calcul.index("-", ind)
        if ind < fin-1 and EstNombre(calcul[ind+1]):
            n = eval(calcul[ind+1])
            if calcul[ind+1][0] == "(" or isinstance(n, Polynome):
#                or (isinstance(n, Polynome) and (len(n)>1 or n.monomes[0][0]<0)):
                debut = ind
                break
        else:
            ind += 1
    for c in range(calcul[ind+1:].count("+")):
        ind = calcul.index("+", ind)
        if ind < fin-1 and EstNombre(calcul[ind+1]) and calcul[ind+1][0] == "(":
            if debut is not None: debut = min(debut, ind)
            else: debut = ind
            break
        else:
            ind += 1
    if debut is not None and (debut+2 == fin or (debut+2<fin and
        calcul[debut+2] != "**")):
        return (debut, debut+2)
    else: return None

def post_polynomes(l_calcul):
    """Réduit les polynômes de la liste de calculs"""
    for k in range(len(l_calcul)):
        if 'Polynome(' in l_calcul[k]:
            p = eval(l_calcul[k])
            if Polynome.reductible(p):
                p = Polynome.reduction_detaillee(p)
                if isinstance(p, Polynome): l_calcul[k] = repr(p)
                else:
                    if isinstance(p, basestring): p = splitting(p)
                    if (k and l_calcul[k-1] in "*-") or (k<len(l_calcul)-1 and l_calcul[k+1] in "**"):
                        p.insert(0, "(")
                        p.append(")")
                    l_calcul[k:k+1] = p
    return l_calcul

def effectue_calcul(calcul):
    """Effectue une étape du calcul en respectant les priorités
    @calcul: list"""
    serie = (recherche_parentheses, recherche_puissance, recherche_produit,
            recherche_neg, recherche_somme)
    result, post, break_somme = [], "", False
    for recherche in serie:
        if break_somme and recherche == recherche_somme: break
        if calcul: test = recherche(calcul)
        else: test = None
        while test:
            pre = calcul[:test[0]]
            calc = calcul[test[0]:test[1]]
            post = calcul[test[1]:]
            # On essaie d'utiliser les priorités sur la première partie du
            # calcul pour raccourcir la résolution de l'expression
            if test != recherche_neg and pre:
                # Si on simplifie les écritures, on n'essaie pas les sommes
                tmp = effectue_calcul(pre)
                if tmp and result and result[-1] not in "+-**/(" and tmp[0][0]\
                    not in "+-*/)":
                    # un signe + si l'expression en a besoin
                    result.append("+")
                # On ajoute le résultat ou l'expression
                if tmp: result.extend(tmp)
                else: result.extend(pre)
            else:
                result.extend(pre)
            if recherche == recherche_parentheses:
                # On supprime les parenthèses autour de l'expression
                calc = calc[1:-1]
                # On réduit directement les calculs numériques dans une
                # expression littérale
                sol = effectue_calcul(calc)
                if len(sol)>1:
                    sol.insert(0, "(")
                    sol.append(")")
                elif sol and isinstance(eval(sol[0]), (int, float)) and post and post[0] == "*" and \
                        'Polynome(' in post[1] and len(eval(post[1])) == 1 and \
                        eval(post[1])[0][0] == 1:
                            # Sans doute une factorisation de sommes de polynômes
                            sol = [repr(eval(sol[0])*eval(post [1]))]
                            post = post[2:]
                            #Suppression des parenthèses autour de ((9.0-80.0)*Polynome("x")) devenu (Polynome("-71.0x"))
                            if post and result and post[0] == ")" and result[-1] == "(" :
                                result,  post = result[:-1],  post[1:]
            else:
                # Transforme les réels en polynômes afin de permettre les opérations
                bpoly = False
                for k in range(len(calc)):
                    if not bpoly and calc[k][:9] == "Polynome(": bpoly = True
                    elif bpoly and calc[k] not in "**/-+()" and calc[k-1] != "**" and isinstance(eval(calc[k]), (float, int)):
                        calc[k] = "Polynome([[%s, 0]])" % eval(calc[k])
                sol = eval("".join(calc))
                if isinstance(sol, basestring): sol = splitting(sol)
                elif isinstance(sol, (int, float)): sol = [str(sol)]
                elif isinstance(sol, Polynome): sol = [repr(sol)]
                else : raise valueError(u"Le résultat a un format inattendu")
            if recherche == recherche_neg:
                # Ajoute le "+" ou sépare le "-":
                # "1-(-9)" => "1 + 9" et "1+(-9)" => "1 - 9"
                if sol[0][0] == "-": sol = ["-", sol[0][1:]]
                else: sol = ["+", sol[0]]
            if recherche == recherche_produit and len(sol) > 1:
                # Ajoute les parenthèses si produit précédé d'un "-" ou "*"
                # ou suivi d'un "*"
                if (result and result[-1] in "*-") or (post and post[0] == "*"):
                    sol.insert(0, "(")
                    sol.append(")")
            # Si @sol est négatif et @result se termine par un "+", on l'enlève
            if result and result[-1] == "+" and sol and sol[0][0] == "-":
                    result[-1] = "-"
                    sol[0] = sol[0].lstrip("-")
            result.extend(sol)
            calcul = post
            if calcul: test = recherche(calcul)
            else: test = None
            if post and recherche == recherche_neg: break_somme=True
    result.extend(post)
    if not result: result = calcul
    tmp = "".join(result)
#    if cherche_polynome(tmp, 0):
#        index = tmp.find("+-")
#        while index >=0 :
#            nb = cherche_decimal(tmp, index+2)
#            tmp = tmp[:index+1] + "(-" + nb + ")" + tmp[index+2+len(nb):]
#            index = tmp.find("+-")
#        result = splitting(tmp)
    return result

def priorites(calcul):
    """Effectue un enchaînement d'opérations contenues dans calcul en
    respectant les priorités et en détaillant les étapes.
    @calcul: str"""
#    if cherche_polynome(calcul, 0)>=0: litteral = True
#    else: litteral = False
    calcul = splitting(calcul)
    solution = []
    while len(calcul)>1:
        s= effectue_calcul(calcul)
        if s:
            s = post_polynomes(s)
            solution.append(s)
        calcul = [k for k in s] # dissocie calcul et s
        if len(calcul) == 1 and post_polynomes(calcul) != s:
            # Finit la réduction du polynôme résultat
            solution.append(calcul)
    return solution

def texify(liste_calculs):
    """Convertit une liste de chaînes de caractères 'liste_calculs' contenant
    des polynômes en liste de chaînes de caractères au format TeX"""
    ls = []
    for calcul in liste_calculs:
        if isinstance(calcul,  basestring): calcul = splitting(calcul)
        s = ""
        puiss = 0
        for i in range(len(calcul)):
            el = calcul[i]
            if el[:9] == "Polynome(":
                # Doit-on entourer ce polynôme de parenthèses ?
                p = eval(el)
                if i+1 < len(calcul): q = calcul[i+1]
                else: q= ""
                if (s and s[-1] in "*-" and (len(p)>1 or p.monomes[0][0]<0)) \
                    or (q and q == "*" and len(p)>1) \
                    or ((len(p)>1 or p.monomes[0][0]!=1) and q and q == "**"):
                    s += "(" + str(p) +")"
                elif s and s[-1] == "+" and p.monomes[0][0]<0:
                    s = s[:-1]
                    s += str(p)
                else:
                    s += str(p)
            elif EstNombre(el):
                if el[0]=="(": s += "(" + decimaux(el[1:-1]) + ")"
                else: s += decimaux(el)
            elif el == "**":
                s += "**{"
                puiss += 1
            elif el == "(":
                if puiss: puiss += 1
                s += "("
            elif el == ")":
                if puiss: puiss -= 1
                s += ")"
            else :
                # "+", "-", "*", "/"
                s += el
            if puiss == 1 and s[-1] != "{":
                puiss = 0
                s += "}"
        s = s.replace("**{", "^{")
        s = s.replace("(", "\\left( ")
        s = s.replace(")", "\\right) ")
        s = s.replace("*", "\\times ")
        s = s.replace("/", "\\div ")
        if not ls or s != ls[-1]:
            ls.append(s)
    return ls

#----------------------------------------------------------------------
# Pyromaths : FICHIERS DE TESTS
#----------------------------------------------------------------------
def valeurs(n, polynomes=0, entier=1):
    """Renvoie une chaîne de caractères contenant un calcul aléatoire de
    n nombres, qui sont soit des entiers, soit des décimaux, soit des polynômes."""
    import random
    def deci(entier):
        if entier:
            return random.randrange(-11, 11)
        else:
            return random.randrange(-110, 110)/10.
    def poly(entier):
        long = random.randrange(3)
        degre=[0, 1, 2]
        p = [[deci(entier), degre.pop(random.randrange(len(degre)))] for i in range(long+1)]
        p = Polynome(p, "x")
        while p == Polynome([[0, 0]], "x"):
            degre=[0, 1, 2]
            p = [[deci(entier), degre.pop(random.randrange(len(degre)))] for i in range(long+1)]
            p = Polynome(p, "x")
        if len(p) == 1 and Polynome.degre(p) == 0:
            p = p[0][0]
            if p < 0: p = "(" + str(p) +")"
            else: p = str(p)
        else: p = repr(p)
        return p
    if polynomes: nb=poly
    else: nb=deci
    nbp = []
    lo = ['+','*','-']
    for i in range(n):
        if not i:
            s = "%s" % nb(entier)
        else:
            if nbp and nbp[-1] > 2 and not random.randrange(2):
                s += ')'
                nbp.pop(-1)
            s += lo[random.randrange(3)]
            if nbp:
                for cpt in range(len(nbp)):
                    nbp[cpt] += 1
            a = random.randrange(3)
            if s[-1] in '*-/' and i<=n-2 and not a:
                s += '('
                nbp.append(1)
            nombre = nb(entier)
            if nombre < 0: s += "(" + str(nombre) + ")"
            else: s += str(nombre)
            if not polynomes: #TODO: tester les puissances avec les polynômes
                a = random.randrange(10)
                if (not nbp or nbp[-1]>1) and not a:
                    s += "**2"
    while nbp:
        s += ')'
        nbp.pop(-1)
    return s

def test_entiers(nbval, polynomes, entiers):
    for c in range(10000):
        a = valeurs(nbval, polynomes, entiers)
        try:
            r = priorites(a)
            texify(r)
        except:
            print(a)
            break
        else:
            if not polynomes:
                print u"%s ème calcul" % (c+1), a, " = ", eval(a), u" en %s étapes"%(len(r))
                if eval(a) != eval("".join(r[-1])):
                    print a, " = ", eval(a)
                    print r
                    break
            if polynomes:
                from sympy import Symbol, expand
                x = Symbol('x')
                print u" : %s ème calcul en %s étapes" % (c+1, len(r))
                if not len(r):
                    print a
                    break
                else:
                    t0=sympyfy([a])[0]
                    t1=sympyfy(r[-1])
                    if isinstance(eval(t1[0]), (int, float)) : t1[0] = "%s+x-x" % (t1[0])
                    if isinstance(eval(t0), (int, float)) : t0 = "%s+x-x" % (t0)

                    if eval(t0).expand()!=eval(t1[0]).expand():
                        print a, "=", t1
                        print "\n".join(sympyfy(r))

                        print u"Devrait être : ", eval(t0).expand()
                        break

def sympyfy(liste_calculs):
    """Convertit une liste de chaînes de caractères 'liste_calculs' contenant
    des polynômes en liste de chaînes de caractères au format Sympy"""
    ls = []
    for calcul in liste_calculs:
        if isinstance(calcul,  basestring): calcul = splitting(calcul)
        s = ""
        puiss = 0
        for i in range(len(calcul)):
            el = calcul[i]
            if el[:9] == "Polynome(":
                # Doit-on entourer ce polynôme de parenthèses ?
                p = eval(el)
                if i+1 < len(calcul): q = calcul[i+1]
                else: q= ""
                if (s and s[-1] in "*-" and (len(p)>1 or p.monomes[0][0]<0)) \
                    or (q and q == "*" and len(p)>1) \
                    or ((len(p)>1 or p.monomes[0][0]!=1) and q and q == "**"):
                    s += "(" + str(p) +")"
                elif s and s[-1] == "+" and p.monomes[0][0]<0:
                    s = s[:-1]
                    s += str(p)
                else:
                    s += str(p)
            elif EstNombre(el):
                if el[0]=="(": s += "(" + decimaux(el[1:-1]) + ")"
                else: s += decimaux(el)
            elif el == "**":
                s += "**{"
                puiss += 1
            elif el == "(":
                if puiss: puiss += 1
                s += "("
            elif el == ")":
                if puiss: puiss -= 1
                s += ")"
            else :
                # "+", "-", "*", "/"
                s += el
            if puiss == 1:
                puiss = 0
                s += "}"
        s = s.replace("\\,x", "*x")
        s = s.replace("\\,", "")
        s = s.replace("**{", "**")
        s = s.replace("^{", "**")
        s = s.replace("}", "")
#        s = s.replace("(", "\\left( ")
#        s = s.replace(")", "\\right) ")
#        s = s.replace("**{", "^{")
#        s = s.replace("*", "\\times ")
#        s = s.replace("/", "\\div ")
        if not ls or s != ls[-1]:
            ls.append(s)
    return ls

def main():
    from sympy import Symbol, expand
    x = Symbol('x')
    test_entiers(nbval=13, polynomes=1, entiers=1)
#    a = valeurs(2, 1, 1)
#    a = 'Polynome([[8, 2]], "x")*Polynome([[3, 1]], "x")-Polynome([[-3, 0], [4, 2], [10, 1]], "x")*Polynome([[-4, 2], [5, 0], [-11, 1]], "x")-Polynome([[3, 2], [-5, 1]], "x")-(Polynome([[3, 2], [-1, 0]], "x")*Polynome([[3, 2], [1, 0]], "x")*Polynome([[-6, 2], [10, 1], [2, 0]], "x"))*Polynome([[-9, 0], [-7, 2], [8, 1]], "x")-Polynome([[-8, 1]], "x")+Polynome([[-8, 2], [4, 0]], "x")+Polynome([[10, 1]], "x")+Polynome([[-9, 2], [6, 1], [7, 0]], "x")'
#    print(a)
##    s= sympyfy([a])
#    print sympyfy(priorites(a))
#    r = texify(priorites(a))
#    print "\\item $A=", texify([a])[0] , "$\\par"
#    for i in r:
#        print "$A=",
#        print "".join(i),
#        print "$\\par"



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#
import random

#
# ------------------- Diagrammes -------------------
#


def exo_diagrammes():
    exo = ["\\exercice"]
    cor = ["\\exercice*"]
    h1 = random.randrange(2) + 5
    h2 = 8
    h3 = random.randrange(2) + 7
    h5 = random.randrange(4)+1
    h6 = random.randrange(3)
    h7 = random.randrange(2)+1
    h4 = 30 - h1 - h2 - h3 - h5 - h6 - h7
    basket = random.randrange(7) + 3
    tennis = random.randrange(7) + 3
    judo = random.randrange(7) + 3
    football = 30 - tennis - basket - judo
    question1 = \
        u"""\\renewcommand{\\arraystretch}{1.8}
        \\item On a demandé aux élèves d'une classe de cinquième combien de temps par semaine était consacré à leur sport favori.\\par
    \\begin{tabular}{|c|c|c|c|c|c|c|c|}\\hline Durée t (en h)&  $0 \\le t < 1$ & $1 \\le t  < 2$ & $2 \\le t  < 3$ & $3 \\le t  < 4$ &  $4 \\le t  < 5$ & $5 \\le t  < 6$ & $6 \\le t  < 7$ \\\\\\hline Effectif & %s & %s & %s & %s & %s & %s & %s \\\\\\hline \\end{tabular}\\par
    À partir de ce tableau, construire un  histogramme pour représenter ces données.\\par""" % (h1, h2, h3, h4, h5, h6, h7)
    question2 = \
        u"""\\item On a demandé aux élèves quel était leur sport préféré. %s élèves préfèrent le basket-ball, %s le tennis, %s le football et %s le judo. Construire un diagramme circulaire représentant cette répartion.\\par""" % (basket, tennis, football, judo)
    exo.append("\\begin{enumerate}")
    exo.append(question1)
    exo.append(question2)
    exo.append("\\end{enumerate}")
    cor.append("\\begin{enumerate}")
    cor.append(question1)
    cor.append(u"""\\begin{minipage}{10cm}
    \\begin{pspicture}(0,-1)(8.5,9.5)
    \\psaxes[showorigin=false]{->}(7.5,8.5)
    \\psset{fillstyle=solid, linewidth=0.5pt}
    \\psframe(0,0)(1,%s)
    \\psframe(1,0)(2,%s)
    \\psframe(2,0)(3,%s)
    \\psframe(3,0)(4,%s)
    \\psframe(4,0)(5,%s)
    \\psframe(5,0)(6,%s)
    \\psframe(6,0)(7,%s)
    \\rput(-0.2,-0.425){$0$}
    \\rput(8.3,0){Durée}
    \\rput(0,8.8){Effectif}
    \\end{pspicture}
    \\end{minipage}
    \\begin{minipage}{6cm}
    Sur l'axe horizontal, on représente les durées en heures et, sur l'axe vertical, on représente les effectifs.
    \\end{minipage}""" % (h1, h2, h3, h4, h5, h6, h7))
    cor.append(question2)
    cor.append(u"L'effectif total est égal à $ %s + %s + %s + %s = 30$. La mesure d'angle d'un secteur circulaire est proportionnelle à l'effectif du sport qu'il représente. Le coefficient de proportionnalité est égal au quotient de l'effectif total par 360\\degre c'est à dire $360 \\div 30=12$.\\par" % (basket, tennis, football, judo))
    cor.append(u"""\\renewcommand\\tabcolsep{10pt}
    \\begin{tabular}{|l|c|c|c|c|c|c}
    \\cline{1-6}
    Sport favori  & Basket-ball & Tennis & Football & Judo & Total &\\rnode{plan1}{}\\\\
    \\cline{1-6}
    Effectif & %s & %s & %s & %s & 30 &\\rnode{plan1}{}\\\\
    \\cline{1-6}
    Mesure (en degré)  & \\bf%s & \\bf%s & \\bf%s & \\bf%s & 360 &\\rnode{plan2}{}\\\\
    \\cline{1-6}
    \\end{tabular}
    \\ncbar{->}{plan1}{plan2}\\Aput{$\\times 12$}\\par
    \\begin{minipage}{6cm}
    En utilisant les mesures d'angles obtenues dans le tableau de proportionnalité, on trace le diagramme circulaire.
    \\end{minipage}""" % (basket, tennis, football, judo, basket*12, tennis*12, football*12, judo*12))
    cor.append(u"""\\begin{minipage}{13cm}
    \\psset{unit=3cm,fillstyle=solid}
    \\pspicture(-1.5,-1)(1,1.5)
    \\pswedge[fillcolor=Bisque]{1}{0}{%s}
    \\pswedge[fillcolor=LightSalmon]{1}{%s}{%s}
    \\pswedge[fillcolor=Chocolate]{1}{%s}{%s}
    \\pswedge{1}{%s}{360}
    \\rput(.6;%s){Basket}
    \\rput(.6;%s){Tennis}
    \\rput(.6;%s){\\white Football}
    \\rput(.6;%s){Judo}
    \\endpspicture
    \\end{minipage}""" %(basket*12, basket*12, basket*12+tennis*12, basket*12+tennis*12, basket*12+tennis*12+football*12, basket*12+tennis*12+football*12, basket*6, basket*12+tennis*6, basket*12+tennis*12+football*6, basket*6+tennis*6+football*6+180 ))
    cor.append(u"\\end{enumerate}")
    return (exo, cor)



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

import ExoPolynome, Vecteurs

def write(f0, f1, exos):
    f0.write("\n")
    f1.write("\n")
    f0.writelines(x + "\n" for x in exos[0])
    f1.writelines(x + "\n" for x in exos[1])

def main(exo, f0, f1):
    modules = (
        ExoPolynome.exo_racines_degre2,
        ExoPolynome.exo_factorisation_degre2,
        ExoPolynome.exo_factorisation_degre3,
        ExoPolynome.exo_tableau_de_signe,
        ExoPolynome.exo_variation,
        ExoPolynome.exo_variation_lim,
        Vecteurs.vecteurs_add,
        )
    write(f0, f1, modules[exo]())



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

from ..classes.Fractions import Fractions  #Fractions de pyromaths
import random
import string
from ..outils import Arithmetique


def fractions_egales():
    exo = ["\\exercice", u"Compléter :", "\\begin{multicols}{4}",
           "  \\begin{enumerate}"]
    cor = ["\\exercice*", u"Compléter :", "\\begin{multicols}{4}",
           "  \\begin{enumerate}"]
    for i in range(8):
        n = d = 1
        while n == d:
            n = random.randrange(1, 11)
            d = random.randrange(2, 11)
        c = random.randrange(2, 11)
        cas = random.randrange(2)
        if cas:
            enonce = [n, d, n * c, d * c]
            solution = [n, d, n * c, d * c]
        else:
            enonce = [n * c, d * c, n, d]
            solution = [n * c, d * c, n, d]
        trou = random.randrange(4)
        enonce[trou] = "\\ldots"
        solution[trou] = "\\mathbf{%s}" % solution[trou]
        if cas:
            solution.insert(2, c)
            solution.insert(1, c)
        else:
            solution.insert(4, c)
            solution.insert(3, c)
        exo.append("\\item $\\dfrac{%s}{%s}=\\dfrac{%s}{%s}$" %
                   tuple(enonce))
        if cas:
            cor.append("\\item $\\dfrac{%s_{(\\times %s)}}{%s_{(\\times %s)}}=\\dfrac{%s}{%s}$" %
                       tuple(solution))
        else:
            cor.append("\\item $\\dfrac{%s}{%s}=\\dfrac{%s_{(\\times %s)}}{%s_{(\\times %s)}}$" %
                       tuple(solution))
    exo.extend(["  \\end{enumerate}", "\end{multicols}\n"])
    cor.extend(["  \\end{enumerate}", "\end{multicols}\n"])
    return (exo, cor)


def sommes_fractions():
    exo = ["\\exercice",
           u"Effectuer les calculs suivants et donner le résultat sous la forme d'une fraction simplifiée :",
           "\\begin{multicols}{4}", "  \\noindent"]
    cor = ["\\exercice*",
           u"Effectuer les calculs suivants et donner le résultat sous la forme d'une fraction simplifiée :",
           "\\begin{multicols}{4}", "  \\noindent"]
    for i in range(8):
        if random.randrange(2):
            s = "+"
        else:
            s = "-"
        n = d = c = n2 = 2
        while Arithmetique.pgcd(n, d) > 1:
            n = random.randrange(1, 11)
            d = random.randrange(2, 11)
        while Arithmetique.pgcd(n2, d * c) > 1:
            c = random.randrange(2, 11)
            n2 = random.randrange(2, 11)
        fr1 = Fractions(n, d)
        fr2 = Fractions(n2, d * c)
        if s == "-" and (fr1 - fr2).n * (fr1 - fr2).d < 0 or s == "+" and \
            random.randrange(2):
            (fr1, fr2) = (fr2, fr1)
        exo.append("\\[ \\thenocalcul = %s%s%s \\]" % (Fractions.TeX(fr1),
                   s, Fractions.TeX(fr2)))
        cor.append("\\[ \\thenocalcul = %s%s%s \\]" % (Fractions.TeX(fr1),
                   s, Fractions.TeX(fr2)))
        if fr1.d < fr2.d:
            cor.append("\\[ \\thenocalcul = %s%s%s \\]" % (Fractions.TeX(fr1,
                       coef=c), s, Fractions.TeX(fr2)))
        else:
            cor.append("\\[ \\thenocalcul = %s%s%s \\]" % (Fractions.TeX(fr1),
                       s, Fractions.TeX(fr2, coef=c)))
        if s == "+":
            fr = fr1 + fr2
        else:
            fr = fr1 - fr2
        frs = Fractions.simplifie(fr)
        if frs.d != fr.d:
            cor.append("\\[ \\thenocalcul = %s \\]" % Fractions.TeX(frs,
                       coef=fr.d // frs.d))
            cor.append("\\[ \\boxed{\\thenocalcul = %s} \\]" %
                       Fractions.TeX(frs))
        else:
            cor.append("\\[ \\boxed{\\thenocalcul = %s} \\]" %
                       Fractions.TeX(fr))
        exo.append("\\stepcounter{nocalcul}%")
        cor.append("\\stepcounter{nocalcul}%")
    exo.append("\end{multicols}\n")
    cor.append("\end{multicols}\n")
    return (exo, cor)


def produits_fractions():
    exo = ["\\exercice",
           u"Effectuer les calculs suivants et donner le résultat sous la forme d'une fraction simplifiée :",
           "\\begin{multicols}{4}", "  \\noindent"]
    cor = ["\\exercice*",
           u"Effectuer les calculs suivants et donner le résultat sous la forme d'une fraction simplifiée :",
           "\\begin{multicols}{4}", "  \\noindent"]
    for i in range(8):
        n1=d1=n2=d2=a=b=2
        while Arithmetique.pgcd(a,b)>1:
            a=random.randrange(1,11)
            b=random.randrange(2,11)
        while Arithmetique.pgcd(n1*a,d1*b)>1:
            n1=random.randrange(1,11)
            d1=random.randrange(2,11)
        while Arithmetique.pgcd(n2*b,d2*a)>1:
            n2=random.randrange(1,11)
            d2=random.randrange(2,11)

        fr1 = Fractions(n1*a, d1*b)
        fr2 = Fractions(n2*b, d2*a)
        exo.append("\\[ \\thenocalcul = %s \\times %s \\]" % (Fractions.TeX(fr1),
                   Fractions.TeX(fr2)))
        cor.append("\\[ \\thenocalcul = %s \\times %s \\]" % (Fractions.TeX(fr1),
                   Fractions.TeX(fr2)))
        fr1s = Fractions.simplifie(fr1)
        fr2s = Fractions.simplifie(fr2)
        if abs(fr1s.d) < abs(fr1.d) or abs(fr2s.d) < abs(fr2.d):
            cor.append("\\[ \\thenocalcul = %s \\times %s \\]" % (Fractions.TeXSimplifie(fr1),
                       Fractions.TeXSimplifie(fr2)))
        fr = fr1s * fr2s
        frs = Fractions.simplifie(fr)
        if abs(frs.d) < abs(fr.d):
            cor.append("\\[ \\thenocalcul = %s \\]" % Fractions.TeXProduit(fr1s,
                       fr2s))
        cor.append("\\[ \\boxed{\\thenocalcul = %s} \\]" % Fractions.TeX(frs))
        exo.append("\\stepcounter{nocalcul}%")
        cor.append("\\stepcounter{nocalcul}%")
    exo.append("\end{multicols}\n")
    cor.append("\end{multicols}\n")
    return (exo, cor)



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#


from . import angles, decimaux, droites, fractions, operations, quotients
from . import symetrie, arrondi, aires, espace
import random


def write(f0, f1, exos):
    f0.write("\n")
    f1.write("\n")
    f0.writelines(x + "\n" for x in exos[0])
    f1.writelines(x + "\n" for x in exos[1])

def main(exo, f0, f1):
    modules = (
        operations.CalculMental,
        decimaux.EcrireNombreLettre,
        decimaux.PlaceVirgule,
        decimaux.EcritureFractionnaire,
        decimaux.Decomposition,
        decimaux.Conversions(1),
        decimaux.Conversions(2),
        decimaux.Conversions(3),
        operations.Operations,
        operations.ProduitPuissanceDix,
        decimaux.ClasserNombres,
        droites.Droites,
        droites.Perpendiculaires,
        droites.Proprietes,
        quotients.Divisible,
        fractions.FractionPartage,
        fractions.QuestionsAbscisses,
        aires.main,
        symetrie.SymetrieQuadrillage,
        angles.MesureAngles,
	espace.main,
        arrondi.ArrondirNombreDecimal
        )
    write(f0, f1, modules[exo]())



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#


from . import fractions, puissances, pgcd, developpements, equations, racines
from . import systemes, proba, affine, geometrie, arithmetique

def write(f0, f1, exos):
    f0.write("\n")
    f1.write("\n")
    f0.writelines(x + "\n" for x in exos[0])
    f1.writelines(x + "\n" for x in exos[1])

def main(exo, f0, f1):
    modules = (
        fractions.tex_fractions,
        puissances.tex_puissances,
        pgcd.tex_pgcd,
        developpements.tex_developpements,
        developpements.tex_factorisations,
        developpements.tex_devfacteq,
        equations.tex_equations,
        racines.tex_racines,
        systemes.tex_systemes,
        affine.affine,
        proba.tex_proba,
        geometrie.tex_thales,
        geometrie.tex_reciproque_thales,
        geometrie.tex_trigo,
        arithmetique.Arithmetique
        )
    write(f0, f1, modules[exo]())



# -*- coding: utf-8 -*-
import re

## TODO Gestion des signes qui se suivent ( 3+-5 ou 3*-5 )

def priorites(calcul, pre = "", post = "", solution = []):
    if not solution: solution = [calcul]
    recherche_parentheses = r"""
    ^(.*)                           #des calculs dans le groupe 0
    (                               #début du groupe 1
      \(                            #commence par une parenthèse ouvrante
      (?:                           #regroupement 0 (non groupe)
        [\+\-]?                     #un éventuel signe
        (?:                         #regroupement 1 (non groupe)
          \(                        #une parenthèse ouvrante
          [\+\-]?                   #un éventuel signe
          \d+\.?\d*(?:e[\+\-]\d+)?  #on cherche un nombre avec une éventuelle
                                    #partie décimale et un éventuel exposant
          \)                        #une parenthèse fermante
        |                           #ou
          \d+\.?\d*(?:e[\+\-]\d+)?  #on cherche un nombre avec une éventuelle
                                    #partie décimale et un éventuel exposant
        |                           #ou
          \w+                       #un littéral
        )                           #fin du regroupement 1
        [\+\-\*/]+                  #un opérateur
      )+                            #fin du regroupement 0 (présent au moins
                                    #un fois)
      [\+\-]?                     #un éventuel signe
      (?:                           #regroupement 0 (non groupe)
          \(                        #une parenthèse ouvrante
          [\+\-]?                   #un éventuel signe
          \d+\.?\d*(?:e[\+\-]\d+)?  #on cherche un nombre avec une éventuelle
                                    #partie décimale et un éventuel exposant
          \)                        #une parenthèse fermante
        |                           #ou
          \d+\.?\d*(?:e[\+\-]\d+)?  #on cherche un nombre avec une éventuelle
                                    #partie décimale et un éventuel exposant
        |                           #ou
        \w+                         #un littéral
      )                             #fin du regroupement 0 présent une seule fois
      \)                            #fini par une parenthèse fermante
    )                               #fin du groupe 1
    (.*)$                           #des calculs pour finir
    """
    test = re.search(recherche_parentheses, calcul, re.VERBOSE)
    while test:
        sous_calcul = test.groups()
        (calcul, solution) = priorites_operations(sous_calcul[1],
                                                  sous_calcul[0],
                                                  sous_calcul[2],
                                                  solution)
        calcul = solution[-1]
        test = re.search(recherche_parentheses, calcul, re.VERBOSE)
    calcul, solution = priorites_operations(calcul, pre, post, solution)
    return(solution)

def traite_operation(operateur, calcul, pre, post, solution):
    pattern = r"""
            (                   #groupe 0
                ^\(*                #le calcul est au début (aux parenthèses près)
                                    #les signes +/- font parti du groupe 2
            |                   #ou
                ^.*?[\+\-]?         #il y a des calculs avant, le 1e signe +/-
                                    #est un opérateur
            )                   #fin du groupe 0
            (                   #groupe 1
                (?:                 #regroupement 1 (non groupe)
                    [\+\-]?                 #un éventuel signe
                    \(                      #une parenthèse ouvrante
                    [\+\-]?                 #un éventuel signe
                    \d+\.?\d*(?:e[\+\-]\d+)? #on cherche un nombre avec une
                                            #éventuelle partie décimale et un
                                            #éventuel exposant
                    \)                     #une parenthèse fermante
                |                   #ou
                    [\+\-]?                 #un éventuel signe
                    \d+\.?\d*(?:e[\+\-]\d+)? #on cherche un nombre avec une
                                            #éventuelle partie décimale et un
                                            #éventuel exposant
                |                   #ou
                    \w+                     #un littéral
                )                   #fin du regroupement 1
                {0}                 #opération
                (?:                 #regroupement 2 (non groupe)
                    [\+\-]?                 #un éventuel signe
                    \(                     #une parenthèse ouvrante
                    [\+\-]?                 #un éventuel signe
                    \d+\.?\d*(?:e[\+\-]\d+)? #on cherche un nombre avec une
                                            #éventuelle partie décimale et un
                                            #éventuel exposant
                    \)                     #une parenthèse fermante
                |                   #ou
                    [\+\-]?                 #un éventuel signe
                    \d+\.?\d*(?:e[\+\-]\d+)? #on cherche un nombre avec une
                                            #éventuelle partie décimale et un
                                            #éventuel exposant
                |                           #ou
                    \w+                     #un littéral
                )                   #fin du regroupement 2
            )                   #fin du groupe 1
            (.*?)$              #tout le reste
            """.format(operateur)
    test = re.search(pattern, calcul, re.VERBOSE)
    while test:
        sous_calcul = test.groups()
        sous_resultat = eval(sous_calcul[1])
        if (sous_calcul[0] and sous_calcul[0][-1] == '(') and \
           (sous_calcul[2][0] and sous_calcul[2][0] == ')') and \
           (post[:2] != '**' or sous_resultat > 0):
            #permet de supprimer les parenthèses
            solution.extend([pre +
                             sous_calcul[0][:-1] +
                             str(sous_resultat) +
                             sous_calcul[2][1:] +
                             post])
        else:
            solution.extend([pre +
                             sous_calcul[0] +
                             str(sous_resultat) +
                             sous_calcul[2] +
                             post])
        calcul = sous_calcul[0] + str(sous_resultat) + sous_calcul[2]
        test = re.search(pattern, calcul, re.VERBOSE)
    return (calcul, pre, post, solution)

def priorites_operations(calcul, pre = "", post = "", solution = []):
    calcul, pre, post, solution = traite_operation("\*\*", calcul, pre,
                                                   post, solution)
    calcul, pre, post, solution = traite_operation("[\*/]", calcul, pre,
                                                   post, solution)
    calcul, pre, post, solution = traite_operation("[\+\-]", calcul, pre,
                                                   post, solution)
    return (calcul, solution)

for element in priorites("(2-5)**2+(5-2)**3+((3*4+5)*2+3*(32-5*7)**2)*5+17*(12-3*5)"):
  print element 
  
#print priorites("((-(-1)+-2)++3)")
#print priorites("4-((-1+-2)+3)")
#print priorites('8-(-5*(2-4))')
#print priorites("1+2+3+4")
#print priorites("-3**+2")
#print priorites("9+8*7")
#print priorites("8+1.3e+2")



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

import math, random

def pgcd(a, b):
    """Calcule le pgcd entre a et b."""
    while b:
        a, b = b, a%b
    return a

def ppcm(a, b):
    """Calcule le ppcm entre a et b."""
    return a*b/pgcd(a, b)

def premier(n):
    """Teste si un nombre est premier."""
    return not [x for x in xrange(2,int(math.sqrt(n)) + 1)
                if n%x == 0]

def eratosthene(n):
    """Etablit la liste des nombres premiers inferieurs a n."""
    return [x for x in xrange(2, n) if premier(x)]


def factor(n):
    """Retourne la liste des facteurs premiers du nombre n."""
    premiers = []
    candidats = xrange(2,n+1)
    candidat = 2
    while not premiers and candidat in candidats:
        if n%candidat == 0 and premier(candidat):
            premiers.append(candidat)
            premiers = premiers + factor(n/candidat)
        candidat += 1
    return premiers


def factorise(n):
    """Retourne la liste des facteurs premiers du nombre n, ainsi que le détail
    de la factorisation."""

    primes = []
    temp = n
    etapes = []
    primes_etapes = []
    limite=int(math.sqrt(n))+1
    candidate = 2
    while (candidate < limite):
        if n % candidate == 0:
            primes.append(candidate)
            primes_etapes.append(str(candidate))
            n = n / candidate
            if n  == 1:
                break
            primes_etapes.append(str(n))
            etapes.append(primes_etapes)
            primes_etapes = primes_etapes[:-1]
        else:
            candidate += 1
    if (n != 1) or (primes == []):
        primes.append(n)
    return (primes, etapes)


def factoriseTex(n):
    """Version LaTeX pour factorise."""

    corrige = ["\\begin{align*}", str(n)]
    etapes = factorise(n)[1]
    primes = factorise(n)[0]

    if len(primes) > 1:
      for i in range(len(etapes)):
        text = ' & = '
        for j in range(len(etapes[i])):
          if j != len(etapes[i]) - 1:
            text += etapes[i][j] + ' \\times '
          else:
            text += etapes[i][j]
        corrige.append(text + '\\\\')
      corrige.append("\\end{align*}")
    else:
      corrige = [str(n) + " est un nombre premier.\\par "]

    return (primes, corrige)


def carrerise(n):
    """Trouve le plus petit facteur par lequel multiplier pour obtenir un carré."""
    if round(math.sqrt(n), 0)==math.sqrt(n):
        return 1
    elif n<=0:
        return n
    else:
        primes = factorise(n)[0]
        q = {}
        for element in primes:
            if (primes.count(element) % 2 == 1):
                q[element]=1
        ncar=1
        for element in q.iterkeys():
            ncar *= element
    return ncar

def combinaison(n, k):
    """renvoie le nombre de combinaisons de n objets pris k à k"""
    if k > n//2:
        k = n-k
    x = 1
    y = 1
    i = n-k+1
    while i <= n:
        x = (x*i)//y
        y += 1
        i += 1
    return x

def signe(a):
    """renvoie 1 si a est>0, -1 si a<0"""
    if a < 0:
        return -1
    else:
        return 1
def valeur_alea(a, b):
    """choisit une valeur comprise entre a et b non nulle"""
    while True:
        alea = random.randrange(a, b + 1)
        if alea != 0:
            return alea

#---------------------------------------------------------------------
# A supprimer dès que quatriemes/developpements.py aura été corrigé
#---------------------------------------------------------------------

def ecrit_tex(file, formule, cadre=None, thenocalcul='\\thenocalcul = ',
              tabs=1):
    """Écrit la ligne dans le fichier"""
    if formule != '':
        if cadre == None or not cadre:
            file.write((u'  \\[ %s%s \\] \n') % (thenocalcul, formule))
        else:
            file.write((u'  \\[ \\boxed{%s%s} \\] \n') % (thenocalcul, formule))
